var cmsApp = angular.module(
    'cmsApp', [
        'pageManagementService',
        'paginationService',
        'ngCookies',
        'ui.bootstrap',
        'ngRoute'
    ]
);

var spinnerFunction = function (data) {
    $('#loading').show();
    return data;
};

cmsApp.directive('cmsNgValidatePassword',function () {
        return {
            require: 'ngModel',
            link: function (scope, elm, attrs, ctrl) {

                ctrl.$parsers.unshift(function (viewValue) {

                    pwdLongEnough = (viewValue && viewValue.length >= 8 ? 'valid' : undefined);
                    pwdShortEnough = (viewValue && viewValue.length <= 20 ? 'valid' : undefined);
                    pwdHasUpperCaseLetter = (viewValue && /[A-Z]/.test(viewValue)) ? 'valid' : undefined;
                    pwdHasLowerCaseLetter = (viewValue && /[a-z]/.test(viewValue)) ? 'valid' : undefined;
                    pwdHasNumber = (viewValue && /\d/.test(viewValue)) ? 'valid' : undefined;

                    if (pwdLongEnough && pwdShortEnough && pwdHasUpperCaseLetter && pwdHasLowerCaseLetter && pwdHasNumber) {
                        ctrl.$setValidity('pwd', true);
                    } else {
                        ctrl.$setValidity('pwd', false);
                    }

                    return viewValue;
                })
            }
        };
    }).directive('cmsNgCheckPassword',function () {
        return {
            require: 'ngModel',
            link: function (scope, elm, attrs, ctrl) {
                ctrl.$parsers.unshift(function (viewValue, $scope) {
                    var noMatch = viewValue != attrs.passwordval;
                    ctrl.$setValidity('noMatch', !noMatch);
                    return viewValue
                })
            }
        }
    }).directive('validImgSrc', ['$rootScope', function ($rootScope) {
        return {
            require: 'ngModel',
            restrict: 'A',
            link: function (scope, elm, attrs, ctrl) {
                if (attrs.ngRequired == "true") {
                    ctrl.$setValidity('required', attrs.ngModel);

                    scope.$watch(attrs.ngModel, function (value) {
                        ctrl.$setValidity('required', value);
                    })
                }
            }
        };
    }]).directive('checkForExistingBranch',function () {
        return {
            require: 'ngModel',
            link: function (scope, elm, attrs, ctrl) {
                ctrl.$parsers.unshift(function (modelValue, $scope) {
                    var branchExists = false;
                    angular.forEach(scope.branchDetails, function (branches, branchType) {
                        angular.forEach(branches, function (branch) {
                            if (branch.branchName == modelValue) {
                                branchExists = true;
                            }
                        });
                    });
                    ctrl.$setValidity('branchExists', !branchExists);
                    return modelValue
                })
            }
        }
    }).directive('checkExistingPageLocation',function () {
        return {
            require: 'ngModel',
            link: function (scope, elm, attrs, ctrl) {
                ctrl.$parsers.unshift(function (modelValue, $scope) {
                    var locationExists = false;
                    var typedLocation = modelValue;
                    if (typedLocation.length > 1 && typedLocation.charAt(typedLocation.length - 1) == '/') {
                        typedLocation = typedLocation.substring(0, typedLocation.length - 1);
                    }
                    if (scope.existingPageLocations.indexOf(typedLocation) >= 0) {
                        locationExists = true;
                    }
                    ctrl.$setValidity('locationExists', !locationExists);
                    return modelValue
                })
            }
        }
    }).directive('newLineFormatter',function () {
        return {
            restrict: 'A',
            require: 'ngModel',
            link: function (scope, element, attrs, ngModel) {
                if (ngModel) {
                    ngModel.$parsers.push(function (value) {
                        return value != undefined ? value.replace(new RegExp('\n', 'g'), "<br/>") : undefined;
                    });

                    ngModel.$formatters.push(function (value) {
                        return value != undefined ? value.replace(new RegExp('<br/>', 'g'), "\n") : undefined;
                    });
                }
            }
        };
    }).directive('imgPreview',function () {
        return {
            restrict: 'A',
            require: 'ngModel',
            link: function (scope, element, attrs, ngModel) {
                scope.$watch(attrs.ngModel, function (value) {
                    element.qtip({content: {text: '<img src="' + attrs.title + '"/>', title: attrs.desc}, show: 'click', hide: 'click'});
                });
            }
        };
    }).directive('helpTextContent',function () {
        return {
            restrict: 'A',
            link: function (scope, element, attrs) {
                scope.$watch(attrs.ngModel, function (value) {
                    var title = attrs.banner == undefined ? 'Field description' : attrs.banner;
                    element.qtip({content: {text: attrs.title, title: title}, position: {at: 'top right', my: 'bottom left' }});
                });
            }
        };
    }).config(function ($httpProvider) {
        $httpProvider.defaults.transformRequest.push(spinnerFunction);
         $httpProvider.interceptors.push('myHttpInterceptor');
        $httpProvider.interceptors.push('sessionValidationInterceptor');
    }).factory('sessionValidationInterceptor', function ($q,$window) {
        return {
            'response': function (response) {
                var headers = response.headers();
                if (headers.x_unauthorized) {
                    angular.element('#sessionTimeoutModal').modal({
                        backdrop: 'static',
                        keyboard: false,
                        show: true
                    });
                }
                return response
            }
        }
    }).factory('myHttpInterceptor', function ($q, $window, $injector) {
         return {
                   'request': function(config) {
    return config;
                    },
                'requestError': function(rejection) {
                    return $q.reject(rejection);
                  },
                    'response': function(response) {
                       // same as above
                       var http = $injector.get('$http');
                        if (http.pendingRequests.length == 0) {
                                           $('#loading').hide();
                                       }
                         return response;
                    },

                    'responseError': function(rejection) {
                      var http = $injector.get('$http');
                     if (http.pendingRequests.length == 0) {
                         $('#loading').hide();
                        }
                        return $q.reject(rejection);
                      }
                  };

    });