cmsApp.controller('CompareFileRevisionsController', [
    '$scope',
    '$routeParams',
    '$http',
    '$q',
    '$window',
    '$rootScope',
    '$location',
    '$filter',
    'errorService',
    'jsonEditorService',
    'loggingService',
    'messageService',
    'jsonDiffProcessorService',
    'editorCommonService',
    'commonUtilService',
    'ckEditorCommonService',
    function CompareFileRevisionsController($scope,$routeParams, $http, $q, $window, $rootScope, $location,$filter, errorService, jsonEditorService, loggingService, messageService, jsonDiffProcessorService,editorCommonService,commonUtilService,ckEditorCommonService) {
        $scope.angular = $window.angular;
        $window._testSupport = {
            isDocReady : function() {
                var phase = $rootScope.$root.$$phase;
                return $http.pendingRequests.length == 0 && phase != '$apply' && phase != '$digest';
            }
        };

        $scope.$on("registerJsonEditorCallbackEvent", function(event, registerCallback){
            registerCallback({
                addItem: function(array){
                    renderJsonDiffEditor();
                },
                removeItem: function(array){
                    renderJsonDiffEditor();
                },
                moveUp: function(array){},
                moveDown: function(array){}
            });
        });

        $scope.$parent.pageTitle = "File History Comparison";

        $scope.filePath = $routeParams.filePath;
        $scope.revisionId = $routeParams.revisionId;
        $scope.page = editorCommonService.truncatedFilePathForPreview($scope.filePath);
        $scope.module = editorCommonService.getModuleName($scope.filePath);

        var revisionChangeLogPromise = $http.get('/cms/ajax/git/atomic/readrevisiondetails', {params :{revisionId: $scope.revisionId}});

        var currentContentPromise = $http.get('/cms/edit/fragment', {params :{fragment: '/'+$scope.filePath}});

        var oldFileContentPromise = $http.get('/cms/ajax/git/atomic/readcontent', {params :{filePath: $scope.filePath, revisionId: $scope.revisionId}});

        $q.all([currentContentPromise, oldFileContentPromise, revisionChangeLogPromise]).then(function (results) {
            var currentContent = results[0].data.getFragment.content;
            var oldContent = results[1].data.gitReadContent.content;
            $scope.oldFileContent = oldContent;
            var revisionDetails = results[2].data.gitReadRevision.gitChangeLog;

            if(commonUtilService.isModuleFile($scope.filePath)) {
                loadJsonDiffEditor(currentContent, oldContent, revisionDetails);
            } else if (commonUtilService.isFragment($scope.filePath)) {
                loadWysiWygDiffEditor(currentContent, oldContent, revisionDetails);
            } else if (commonUtilService.isSharedModuleFile($scope.filePath)) {
                loadSharedModuleEditor(currentContent, oldContent, revisionDetails);
            }
        }, errorService.errorHandler);

        $scope.saveModuleJson = function () {
            var deferred = $q.defer();
            saveModule($scope.filePath, deferred.promise);

            deferred.resolve();
        };

        $scope.previewContent = function (moduleForm) {
            var deferred = $q.defer();
            var promise = deferred.promise;

            if(moduleForm.$dirty && moduleForm.$valid) {
               saveModule($scope.filePath, promise);
            }

            promise.then(function(result){
                var correctedFilePath = '/' + $scope.filePath;
                editorCommonService.preview(correctedFilePath);
            });

            deferred.resolve();
        };

        $scope.saveFragment=function () {
         if (CKEDITOR.instances.ckEditorRight.checkDirty()) {
            var content = CKEDITOR.instances.ckEditorRight.getData();
            $http.post("/cms/edit/fragment", {content: content, fragmentPath: $scope.filePath})
                .then(function (result) {
                    messageService.info("Fragment saved. " + $filter('date')(result.data.saveFragment.savedDate.millis, "medium"));
                    loggingService.info('cms-compare-file-revisions-controllerJs', 'Saved fragment ' + $scope.filePath);
                }, errorService.errorHandler);
         }
        }

        $scope.previewFragment=function () {
           $scope.saveFragment();
           editorCommonService.preview("/" + $scope.filePath);
       }

       $scope.copyToRight=function(leftObject) {
           leftObject.meta.rightConflictedObj.value=leftObject.value;
           delete  leftObject.meta.colorScheme;
           delete  leftObject.meta.rightConflictedObj.meta.colorScheme;
           angular.element("#moduleDiffRight").scope().moduleForm.$dirty = true;
       };

       $scope.copyAllChanges=function() {
            angular.element("#moduleDiffRight").scope().jsonReference = null;
            angular.element("#moduleDiffLeft").scope().jsonReference = null;
            loadJsonDiffEditor($scope.oldFileContent, $scope.oldFileContent, $scope.revisionDetails);
            angular.element("#moduleDiffRight").scope().moduleForm.$dirty = true;
       };

       $scope.copyAllFragmentChanges=function() {
            $scope.rightContent = $scope.leftContent;
            CKEDITOR.instances['ckEditorRight'].setData($scope.rightContent);
       };

        $scope.saveSharedModule=function () {
          $http.post('/cms/sharedmoduleedit/ajax/save', {moduleFilePath: $scope.filePath,sharedFilePath: $scope.currentSharedModuleRight})
              .then(function (result) {
                  messageService.info("Saved Module Successfully.");
                  loggingService.info('cms-compare-file-revisions-controllerJs', 'shared module saved ' + $scope.filePath);
                  $scope.sharedModuleForm.$setPristine();
              }, errorService.errorHandler);
        }

        $scope.copyAllSharedModule = function() {
              if ($scope.availableSharedModules.indexOf($scope.currentSharedModuleLeft) != -1) {
                   $scope.currentSharedModuleRight=$scope.currentSharedModuleLeft;
                   $scope.sharedModuleForm.$dirty = true;
              } else {
                   messageService.info("Shared module from revision history, doesn't exists in available shared modules.");
              }
        }

        $scope.previewSharedModule=function () {
              if($scope.sharedModuleForm.$dirty && $scope.sharedModuleForm.$valid) {
                   $scope.saveSharedModule();
              }
              editorCommonService.preview("/" + $scope.filePath);
        }


        function loadJsonDiffEditor(currentContent, oldContent, revisionDetails) {
            currentContent = /module\((.*?),(.*?),\s*({\s*(.|[\n\r])*}\s*)\)\s*$/g.exec(currentContent);
            oldContent = /module\((.*?),(.*?),\s*({\s*(.|[\n\r])*}\s*)\)\s*$/g.exec(oldContent);
            $scope.revisionDetails = revisionDetails;

            var oldContentJson = JSON.parse(oldContent[3]);
            var currentContentJson = JSON.parse(currentContent[3]);

            $http.get('/cms/moduleedit/ajax/modulejsonschema', {params: {moduleFilePath: '/' + $scope.filePath}}).
                then(function (data) {
                    var schemaJson = JSON.parse(data.data.getModuleSchema.schema);

                    $scope.schemaJson =  schemaJson;
                    $scope.oldJsonReference = jsonEditorService.addSchemaAttributesToJsonDoc(oldContentJson, schemaJson);
                    $scope.newJsonReference = jsonEditorService.addSchemaAttributesToJsonDoc(currentContentJson, schemaJson);

                    renderJsonDiffEditor();
                });

//            $scope.moduleDiffComparison = { left: oldContentJson, right:  currentContentJson};
        }

        function loadWysiWygDiffEditor(currentContent, oldContent, revisionDetails) {
            $scope.displayFragmentDiffEditor=true;

            $scope.rightContent = currentContent;
            $scope.leftContent = oldContent;
            $scope.revisionDetails = revisionDetails;

            initializeWysiWygEditor();
        }

        function loadSharedModuleEditor(currentContent, oldContent, revisionDetails) {
                    $scope.displaySharedModuleDiffEditor=true;
                    $scope.rightContent = currentContent;
                    $scope.leftContent = oldContent;
                    $scope.revisionDetails = revisionDetails;

                    $q.all([
                        $http.get('/cms/sharedmoduleedit/ajax/availablesharedfiles', {params: {moduleFilePath: $scope.filePath} })
                    ]).
                        then(function (data) {
                            $scope.availableSharedModules = data[0].data.sharedModules.sharedModulesList;
                            var sharedModuleParsePattern = /#parse\("(.*.vm)"\)/i;
                            $scope.currentSharedModuleLeft = commonUtilService.findMatchingGroups(oldContent,sharedModuleParsePattern)[0];
                            $scope.currentSharedModuleOrig = commonUtilService.findMatchingGroups(currentContent,sharedModuleParsePattern)[0];
                            $scope.currentSharedModuleRight = commonUtilService.findMatchingGroups(currentContent,sharedModuleParsePattern)[0];
                        }, errorService.errorHandler);
        }

        function saveModule(moduleFilePath, promise) {
            var jsonToSave = jsonEditorService.removeSchemaAttributesFromJsonDoc($scope.newJsonReference, {});

            promise = promise.then(function(result){
                return $http.post('/cms/moduleedit/ajax/save', {moduleJson: JSON.stringify(jsonToSave), moduleFilePath: moduleFilePath});
            });

            promise.then(function (result) {
                loggingService.info('cms-compare-file-revision-controllerJs', 'module json saved ' + moduleFilePath );
                messageService.info("Saved Module Successfully.");
                angular.element("#moduleDiffRight").scope().moduleForm.$dirty = false;
            }, errorService.errorHandler);

            return promise;
        }

        function renderJsonDiffEditor() {
            jsonDiffProcessorService.processComparisonObjects($scope.schemaJson, $scope.oldJsonReference, $scope.newJsonReference);

            angular.element("#moduleDiffRight").scope().jsonReference = $scope.newJsonReference;
            angular.element("#moduleDiffLeft").scope().jsonReference = $scope.oldJsonReference;

            $scope.displayModuleDiffEditor=true;

            loggingService.info("cms-compare-file-revision-controllerJs", "presenting module difference for " +$scope.filePath);
        }

        function initializeWysiWygEditor() {
            var editorServicePromise = ckEditorCommonService.ckEditorConfig($scope.filePath);

            editorServicePromise.then(function(result) {
                var readOnlyConfig = {};
                angular.copy(ckEditorCommonService.editorConfig, readOnlyConfig);
                readOnlyConfig.readOnly = true;

                CKEDITOR.replace('ckEditorLeft', readOnlyConfig);
                CKEDITOR.replace('ckEditorRight', ckEditorCommonService.editorConfig);

                $scope.displayFragmentDiffEditor = true;

                loggingService.info("cms-compare-file-revisions-controllerJs", "loading WYSIWYG editor for " +$scope.filePath);
            }, errorService.errorHandler);
        }
    }
]);