cmsApp.controller('ConflictListController', [
    '$scope',
    '$window',
    '$http',
    '$rootScope',
    '$location',
    'errorService',
    'messageService',
    'gitService',
    'jsonEditorService',
    'loggingService',
    'jsonDiffProcessorService',
    '$q',
    'editorCommonService',
    '$filter',
    'commonUtilService',
    'conflictService',
    'ckEditorCommonService',
    'scssService',
    function ($scope, $window, $http, $rootScope, $location, errorService, messageService, gitService, jsonEditorService, loggingService, jsonDiffProcessorService, $q, editorCommonService, $filter, commonUtilService, conflictService, ckEditorCommonService,scssService) {

        $window._testSupport = {
            isDocReady: function () {
                var phase = $rootScope.$root.$$phase;
                return $http.pendingRequests.length == 0 && phase != '$apply' && phase != '$digest';
            }
        };

        /*
         It is possible that someone will refresh the page while doing conflict resolution.
         This is a significant problem because it causes angular to reload all state.
         We don't support reloading the conflict page during the middle of a merge.

         So what happens in a refresh is that conflictData will be null, so we can use that to test for a refresh
         and redirect to the main page.  That page will reset the workspace if we are in a conflict and restart the
         process.
         */
        if (!conflictService.conflictData) {
            $location.path("/menu");
            $scope.$parent.pageTitle = "";
            return;
        }

        loggingService.info("cms-conflict-list-controllerJs", "We have experience a merge conflict, attempting to resolve.");


        $rootScope.menuDisabled = true;
        $scope.$parent.pageTitle = "Conflict Resolution";
        $scope.text = conflictService.text;
        $scope.oursReference = conflictService.oursReference;
        $scope.theirsReference = conflictService.theirsReference;

        //TODO - possibly clean this a bit
        var autoGeneratedFiles = filterFiles(conflictService.conflictData.fileNames);
        $scope.conflictFiles = conflictService.conflictData.fileNames;

        loggingService.info("cms-conflict-list-controllerJs", "Conflict found in " +$scope.conflictFiles);

        $scope.currentFile = null;
        var currentFileIndex = -1;

        $scope.onComplete = complete;
        $scope.onOverwrite = overwrite;
        $scope.onLoadEditor = loadEditor;

        initialiseModuleDiffEditor();
        initialiseFragmentDiffEditor();

        function loadEditor(file, index) {

            $scope.currentFile = file;
            currentFileIndex = index;

            if (commonUtilService.isFragment(file)) {
                $scope.loadFragmentEditor();
            } else {
                $scope.loadModuleEditor();
            }
        }

        function overwrite(file, index) {
            $scope.displayModuleDiffEditor = false;
            $scope.displayFragmentDiffEditor = false;

            gitService
                .setErrorMessage("Revert of file during conflict resolution failed.")
                .checkoutFile(file)
                .add(file)
                .then(function (result, control) {
                    loggingService.info("cms-conflict-list-controllerJs", "reverted file " + file);
                    $scope.conflictFiles.splice(index, 1);
                    if (commonUtilService.isModuleFile(file)) {
                        $http.post('/cms/moduleedit/ajax/regeneratescss', [file]).then(function () {
                            loggingService.info("cms-conflict-list-controllerJs", "regenerated scss for file " + file);
                            control.accept();
                        }, errorService.errorHandler);
                    } else {
                        control.accept();
                    }
                });
        }

        function complete() {
           var gruntPromise =  scssService.runGrunt(
                'www',
                function(gruntResult){
                    loggingService.info("cms-conflict-list-controllerJs", "executed grunt process");
                   var flow = gitService
                                           .setErrorMessage("Concluding conflict resolution failed.");
                               // autoGeneratedFiles contains files we generate.  if there is a conflict in one of them, there must be a conflict in the
                               // module file.  We must have resolved that, so we must have regenerated the scss file and css file.  So by this stage
                               // we have a new scss file which we can just add.
                               for (var i = 0; i < autoGeneratedFiles.length; i++) {
                                   flow.add(autoGeneratedFiles[i]);
                               }

                               flow.commit('CMS resolved merge conflicts as ' + $rootScope.rootScopeCurrentUser.user)
                                   .push()
                                   .then(function (result, control) {
                                       loggingService.info("cms-conflict-list-controllerJs", "concluding conflict resolution");
                                       $rootScope.menuDisabled = false;
                                       conflictService.conflictCallback();
                                       control.accept();
                                   });
                }, errorService.errorHandler);
        }

        function initialiseFragmentDiffEditor() {
            $scope.displayFragmentDiffEditor = false;
            $scope.leftContent = null;
            $scope.rightContent = null;
            $scope.filePath = null;

            $scope.onFragmentSave = function () {
                fragmentSave(true);
            };

            $scope.loadFragmentEditor = function () {
                $scope.displayModuleDiffEditor = false;
                $http
                    .get('/cms/ajax/conflict/readfile', {params: {file: $scope.currentFile}})
                    .then(function (result) {
                        $scope.leftContent = result.data.conflictReadFile.theirs;
                        $scope.rightContent = result.data.conflictReadFile.ours;

                        initializeWysiWygEditor();

                    });
            };

            function fragmentSave(addToGit) {
                var content = CKEDITOR.instances.ckEditorRight.getData();
                $http.post("/cms/edit/fragment", {content: content, fragmentPath: $scope.currentFile})
                    .then(function (result) {
                        messageService.info("Fragment saved. " + $filter('date')(result.data.saveFragment.savedDate.millis, "medium"));
                        loggingService.info('cms-conflict-list-controllerJs', 'Saved fragment ' + $scope.currentFile);

                        if (addToGit) {
                            gitService
                                .setErrorMessage("Resolution of file during conflict resolution failed.")
                                .add($scope.currentFile)
                                .then(function (result, control) {
                                    loggingService.info("cms-conflict-list-controllerJs", "resolved file " + $scope.currentFile);
                                    $scope.conflictFiles.splice(currentFileIndex, 1);
                                    $scope.displayFragmentDiffEditor = false;
                                    $scope.currentFile = null;
                                    currentFileIndex = -1;
                                    control.accept();
                                });
                        }

                    }, errorService.errorHandler);
            }

            function initializeWysiWygEditor() {
                var editorServicePromise = ckEditorCommonService.ckEditorConfig($scope.currentFile);

                editorServicePromise.then(function(result) {
                    var readOnlyConfig = {};
                    angular.copy(ckEditorCommonService.editorConfig, readOnlyConfig);
                    readOnlyConfig.readOnly = true;

                    if (CKEDITOR.instances.ckEditorLeft == undefined) {
                                        CKEDITOR.replace('ckEditorLeft', readOnlyConfig);
                    }
                    if (CKEDITOR.instances.ckEditorRight == undefined) {
                        CKEDITOR.replace('ckEditorRight', ckEditorCommonService.editorConfig);
                    }
                    CKEDITOR.instances['ckEditorRight'].setData($scope.RightContent);
                    CKEDITOR.instances['ckEditorLeft'].setData($scope.leftContent);

                    $scope.displayFragmentDiffEditor = true;

                    loggingService.info("cms-conflict-list-controllerJs", "loading WYSIWYG editor for  " +$scope.currentFile);
                }, errorService.errorHandler);
            }
        }

        function initialiseModuleDiffEditor() {
            $scope.displayModuleDiffEditor = false;
            $scope.ourJsonReference = null;
            $scope.theirJsonReference = null;
            $scope.schemaJson = null;

            $scope.onModuleSave = moduleSave;
            /*
             When the user clicks on various buttons that affect arrays or panels such as adding elements or reordering
             elements, the action is handled by the json editor service.
             It emits events so that we can re-render the diff editor that contains the two json editors.
             This causes the diff editor to update colors that show differences.
             */
            $scope.$on("registerJsonEditorCallbackEvent", function (event, registerCallback) {
                registerCallback({
                    addItem: function (array) {
                        renderJsonDiffEditor();
                    },
                    removeItem: function (array) {
                        renderJsonDiffEditor();
                    },
                    moveUp: function (array) {
                    },
                    moveDown: function (array) {
                    }
                });
            });

            /*
             This function is used by the diff editor to transfer content from the left json editor to the right json editor.
             */
            $scope.copyToRight = function (leftObject) {
                leftObject.meta.rightConflictedObj.value = leftObject.value;
                delete leftObject.meta.colorScheme;
                delete leftObject.meta.rightConflictedObj.meta.colorScheme;
            };

            $scope.loadModuleEditor = function () {
                $scope.displayFragmentDiffEditor = false;
                angular.element("#moduleDiffRight").scope().jsonReference = null;
                angular.element("#moduleDiffLeft").scope().jsonReference = null;


                var promise = $http.get('/cms/ajax/conflict/readfile', {params: {file: $scope.currentFile}});

                promise.then(function (result) {

                    var theirsMatch = /module\((.*?),(.*?),\s*({\s*(.|[\n\r])*}\s*)\)\s*$/g.exec(result.data.conflictReadFile.theirs);
                    var oursMatch = /module\((.*?),(.*?),\s*({\s*(.|[\n\r])*}\s*)\)\s*$/g.exec(result.data.conflictReadFile.ours);

                    var theirJson = JSON.parse(theirsMatch[3]);
                    var ourJson = JSON.parse(oursMatch[3]);

                    $http.get('/cms/moduleedit/ajax/modulejsonschema', {params: {moduleFilePath: '/' + $scope.currentFile}}).
                        then(function (result) {
                            var schemaJson = JSON.parse(result.data.getModuleSchema.schema);
                            $scope.schemaJson = schemaJson;
                            $scope.theirJsonReference = jsonEditorService.addSchemaAttributesToJsonDoc(theirJson, schemaJson);
                            $scope.ourJsonReference = jsonEditorService.addSchemaAttributesToJsonDoc(ourJson, schemaJson);


                            renderJsonDiffEditor();
                        });
                }, errorService.errorHandler);
            };

            function moduleSave() {
                $scope.displayModuleDiffEditor = false;

                var jsonToSave = jsonEditorService.removeSchemaAttributesFromJsonDoc($scope.ourJsonReference, {});
                $http.post('/cms/moduleedit/ajax/save', {moduleJson: JSON.stringify(jsonToSave), moduleFilePath: $scope.currentFile,runGrunt:false})
                    .then(function (result) {

                        gitService
                            .setErrorMessage("Resolution of file during conflict resolution failed.")
                            .add($scope.currentFile)
                            .then(function (result, control) {
                                loggingService.info("cms-conflict-list-controllerJs", "resolved file " + $scope.currentFile);
                                $scope.conflictFiles.splice(currentFileIndex, 1);
                                control.accept();
                            });

                    }, errorService.errorHandler);
            }

            function renderJsonDiffEditor() {
                jsonDiffProcessorService.processComparisonObjects($scope.schemaJson, $scope.theirJsonReference, $scope.ourJsonReference);

                angular.element("#moduleDiffRight").scope().jsonReference = $scope.ourJsonReference;
                angular.element("#moduleDiffLeft").scope().jsonReference = $scope.theirJsonReference;
                loggingService.info("cms-conflict-list-controllerJs", "presenting module difference for " +$scope.currentFile);
                $scope.displayModuleDiffEditor = true;
            }


        }

        /*
         Go through list of conflicted files and remove any that we auto generate.
         Returns a list of auto generated files that we will later use to add before commit.
         */
        function filterFiles(conflictFiles) {
            var autoGeneratedFiles = [];
            for (var i = 0; i < conflictFiles.length; i++) {
                var filename = conflictFiles[i];
                if ((filename.substring(filename.length - 5) === ".scss" && filename.indexOf("/scss/pages/") > -1) || filename.substring(filename.length - 4) === ".css") {
                    if (autoGeneratedFiles.indexOf(filename) == -1) {
                        autoGeneratedFiles.push(filename);
                    }
                }
            }
            angular.forEach(autoGeneratedFiles, function (scssFile) {
                if (conflictFiles.indexOf(scssFile) >= 0) {
                    conflictFiles.splice(conflictFiles.indexOf(scssFile), 1);
                }
            });
            return autoGeneratedFiles;
        }

    }]);
