cmsApp.controller('EditPageController', [
    '$scope',
    '$http',
    '$window',
    '$location',
    '$rootScope',
    '$q',
    '$routeParams',
    'errorService',
    'messageService',
    'gitService',
    'loggingService',
    'editorCommonService',
    'conflictService',
    'scssService',
function ($scope,
          $http,
          $window,
          $location,
          $rootScope,
          $q,
          $routeParams,
          errorService,
          messageService,
          gitService,
          loggingService,
          editorCommonService,
          conflictService,
          scssService
    ) {
    $window._testSupport = {
        isDocReady : function() {
            var phase = $rootScope.$root.$$phase;
            return $http.pendingRequests.length == 0 && phase != '$apply' && phase != '$digest';
        }
    };
    init();

    function init(){
        if ($rootScope.rootScopeCurrentBranch == 'master'){
            messageService.error("Editing files on master is prohibited.  Please create a branch.");
            $location.path("/");
            return;
        }

        messageService.cleanupOldMessages();

        var file = $routeParams.fragment;
        $scope.file = file;
        var matches = /(.*)\/(.*?)$/.exec(file);
        $scope.filePath = matches[1];
        $scope.fileName = matches[2];

        if (new Date().getTime() - $scope.$parent.lastUpdatedRepo.getTime() > 200){
            $scope.$parent.lastUpdatedRepo = new Date();
            gitUpdate();
        } else {
            loadEditor();
        }
    }

    function gitUpdate(){

        var branchName = gitService.reference();

        gitService
            .setErrorMessage("Updating content before editing failed.")
            .currentBranch()
            .then(function(result, control){
                branchName.value = result.data.gitCurrentBranch.name;
                control.accept();
            })
            .status()
            .then(function(result, control){
                if (!result.data.gitStatus.clean){
                    loadEditor();
                    control.reject();
                } else {
                    control.accept();
                }
            })
            .pull()
            .then(function(result, control){
                if (result.data.gitConflict){

                    conflictService.handleConflict(result.data.gitConflict,
                        "Conflict while merging branch changes.",
                        branchName.value,
                        "remotes/origin/" + branchName.value,
                        conflictCallback);

                    control.reject();
                } else {
                    scssService.onPullRegenerateScss(
                                result,
                                function(){control.accept()},
                                function(error){control.reject();errorService.errorHandler(regScssRes.error);});
                }

            })
            .merge('remotes/origin/master')
            .then(function(result, control){
                if (result.data.gitConflict){

                    conflictService.handleConflict(result.data.gitConflict,
                        "Conflict while merging changes from production.",
                        branchName.value,
                        "remotes/origin/master",
                        conflictCallback);

                    control.reject();
                } else {
                    messageService.info("We have updated your branch from its remote branch and from remote master.");
                    loggingService.info("cms-edit-controllerJs", "updated local repo for " + branchName.value);
                    loadEditor();
                    control.accept();
                }
            });
    }

    function conflictCallback(){
        $location.path("/edit");
    }

    function loadEditor() {
        var file = $scope.file;
        $scope.editor = editorCommonService.editorType(file);
        switch ($scope.editor){
            case WYSIWYG_EDITOR: {
                $location.search('fragment',file).path('/wysiwyg-edit');
                break;
            }
            case MODULE_EDITOR: {
                $location.search('fragment',null).search('module',file).path('/module-edit');
                break
            }
            case SHARED_MODULE_EDITOR : {
                $location.search('fragment',null).search('module',file).path('/shared-module-edit');
                break;
            }
            default :  {
                messageService.error("Cannot edit file: " + file);
                $location.search("module",null).path("/page-management");
            }
        }
    }

}]);