cmsApp.controller('EditModuleController', [
    '$scope',
    '$http',
    '$window',
    '$rootScope',
    '$q',
    '$location',
    'errorService',
    'messageService',
    '$routeParams',
    'jsonEditorService',
    'loggingService',
    'editorCommonService',
    function ($scope, $http, $window, $rootScope, $q, $location, errorService, messageService, $routeParams, jsonEditorService, loggingService, editorCommonService) {

        $window._testSupport = {
            isDocReady: function () {
                var phase = $rootScope.$root.$$phase;
                return $http.pendingRequests.length == 0 && phase != '$apply' && phase != '$digest';
            }
        };

        var moduleFilePath = $routeParams.module;
        $scope.$parent.pageTitle = "Edit Module";
        $scope.file = $routeParams.module;
        loggingService.info('cms-edit-module-controllerJs','loading module editor for' +$scope.file);

        var schemaJson = null;
        $http.get('/cms/moduleedit/ajax/modulejsonschema', {params: {moduleFilePath: moduleFilePath}})
            .then(function(result){
                schemaJson = JSON.parse(result.data.getModuleSchema.schema);
                return $http.get('/cms/moduleedit/ajax/modulejson', {params: {moduleFilePath: moduleFilePath} });

            }, function(error){
                errorService.errorHandler(error, "Error downloading module json: " + moduleFilePath);
            })
            .then(function(result){
                $scope.jsonReference = jsonEditorService.addSchemaAttributesToJsonDoc(JSON.parse(result.data.getModuleJson.json), schemaJson);
            }, function(error){
                errorService.errorHandler(error, "Error downloading module schema: " + moduleFilePath);
            });

        $scope.saveModuleJson = function (moduleFilePath) {
            var deferred = $q.defer();
            saveModule(moduleFilePath, deferred.promise);

            deferred.resolve();
        };

        $scope.previewContent = function (moduleFilePath) {
            var deferred = $q.defer();
            var promise = deferred.promise;
            if($scope.moduleForm.$dirty && $scope.moduleForm.$valid) {
                saveModule(moduleFilePath, promise);
            }

            promise.then(function(result){
                editorCommonService.preview(moduleFilePath);
            });

            deferred.resolve();
        };

        $scope.close = function(){
            editorCommonService.close();
        };

        $scope.fileHistory = function () {
            editorCommonService.showFileHistory($scope.file);
        };

        function saveModule(moduleFilePath, promise) {
            var jsonToSave = jsonEditorService.removeSchemaAttributesFromJsonDoc($scope.jsonReference, {});

            promise = promise.then(function(result){
                return $http.post('/cms/moduleedit/ajax/save', {moduleJson: JSON.stringify(jsonToSave), moduleFilePath: moduleFilePath});
            });

            promise.then(function (result) {
                    loggingService.info('cms-edit-module-controllerJs', 'module json saved ' + moduleFilePath );
                    messageService.info("Saved Module Successfully.");
                    $scope.moduleForm.$setPristine();
            }, errorService.errorHandler);

            return promise;
        }
}]);
