cmsApp.controller('GlobalController', [
    '$scope',
    '$http',
    '$window',
    '$rootScope',
    '$cookies',
    '$q',
    '$location',
    'errorService',
    'messageService',
    'gitService',
    'loggingService',
    '$routeParams',
    'userService',
    'conflictService',
    'scssService',
function ($scope,
          $http,
          $window,
          $rootScope,
          $cookies,
          $q,
          $location,
          errorService,
          messageService,
          gitService,
          loggingService,
          $routeParams,
          userService,
          conflictService,
          scssService) {

    $window._testSupport = {
        isDocReady : function() {
            var phase = $rootScope.$root.$$phase;
            return $http.pendingRequests.length == 0 && phase != '$apply' && phase != '$digest';
        }
    };

    //This handler closes image popups when they are opened and navigation changes to a different view.
    $rootScope.$on("$locationChangeStart", function(event, next, current) {
        $('div.qtip:visible').qtip('destroy');
    });

    //this function is used by views to convert title's into identifiers for testing purposes
    $scope.makeId = function(string){
        return string.trim().replace(' ','-');
    };


    function initialise() {
        loggingService.info('cms-global-controllerJs','Initializing CMS global controller');

        $rootScope.rootScopeCurrentUser = userService.getUserData();

        $scope.requestedLocationPath = $location.path();

        $scope.pageTitle = "";

        $scope.selectedUser = "";

        $scope.lastUpdatedRepo = new Date();

        gitService
            .setErrorMessage("Checking workspace state after login failed.")
            .exists()
            .then(function(result, control){
                if (result.data.gitExists.exists){
                    updateLocalRepository();
                    loggingService.info("cms-global-controllerJS", "updated local repository");
                } else {
                    cloneNewRepository();
                    loggingService.info("cms-global-controllerJS", "cloned local repository");
                }
            });
    }

    $scope.$on('refreshGlobalCtrl',function(event){
        initialise();
    });

    //this is called from the menu-view.page.vm.
    $scope.init = initialise;

    initialise();


    function cloneNewRepository(){
        gitService
            .setErrorMessage("Cloning workspace after login failed.")
            .clone()
            .then(function(result, control){
                if (result.data.gitClone.alreadyExists){
                    updateLocalRepository();
                } else {
                    $scope.$root.rootScopeCurrentBranch = "master";
                    messageService.info("Your account has been created. Please open Branches from the My Work menu and create or select a branch to work on.");
                }
            });
    }

    function updateLocalRepository(){

        var branchName = null;
        gitService
            .setErrorMessage("Failure while establishing current branch information.")
            .pruneDeletedBranches()
            .then(function(result, control){
                if (result.data.gitPruneDeletedBranches.deletedCurrentBranch){
                    messageService.error("The branch you were on has been merged by another user. You are now on the master branch. Please select or create another branch.");
                }
                control.accept();
            })
            .currentBranch()
            .then(function(result, control){
                branchName = result.data.gitCurrentBranch.name;
                $scope.$root.rootScopeCurrentBranch = branchName;
                control.accept();
            })
            .status()
            .then(function(result, control){
                if(result.data.gitStatus.conflicting.length > 0){

                    //We have encountered a workspace that is in a merging state.
                    //We don't know what journey this is a part of, so we reset and commence normal updates.
                    //The merge should then re-establish itself at the appropriate point, and we will then
                    //have the correct context and know what to do after merging.

                    gitService
                        .setErrorMessage("Failure while attempting to reset current environment after a merge state has been encountered.")
                        .reset()
                        .then(function(result, control){
                            mergeWithRemotes(branchName);
                            control.accept();
                        });

                } else if (result.data.gitStatus.clean){
                    mergeWithRemotes(branchName);
                } else {
                    //we have a dirty environment, so the user must commit before we can do any updates.
                    messageService.error("There are unsaved changes on the current branch.");
                    if($scope.requestedLocationPath.indexOf("conflict-list") != -1) {
                        $scope.pageTitle = "";
                        $rootScope.menuDisabled = false;
                        $location.path("/menu");
                    }
                    validate();
                                    }
                control.accept();
            });
    }

    function mergeWithRemotes(branchName){

        var flow = null;
        if (branchName == "master"){
            flow = gitService
                .setErrorMessage("Failure while updating local master with remote master")
                .fetch();

            commenceMasterMerge(branchName, flow);
        } else {
            flow = gitService
                .setErrorMessage("Failure while updating local branch with remotes")
                .pull()
                .then(function(result, control){
                    if (result.data.gitConflict){

                        conflictService.handleConflict(result.data.gitConflict,
                            "Conflict while merging branch changes.",
                            branchName,
                            "remotes/origin/" + branchName,
                            function(){
                                $scope.pageTitle = "";
                                commenceMasterMerge(branchName, gitService.flow().setErrorMessage("Failure while continuing update after merge with remote branch."));
                            });

                        control.reject();
                    } else {
                        scssService.onPullRegenerateScss(
                                result,
                                function(){control.accept()},
                                function(error){
                                    control.reject();
                                    errorService.errorHandler(error);
                                });
                    }
                });

            commenceMasterMerge(branchName, flow);
        }

    }

    function commenceMasterMerge(branchName, flow){

        flow.merge('remotes/origin/master')
            .then(function(result, control){
                 if (result.data.gitConflict){
                     conflictService.handleConflict(result.data.gitConflict,
                         "Conflict while merging changes from production.",
                         branchName,
                         "remotes/origin/master",
                         function(){
                             $scope.pageTitle = "";
                             finish(gitService.flow().setErrorMessage("Failure while continuing update after merge with remote master."));
                         });

                     control.reject();
                 } else {
                    control.accept();
                 }
            });

        finish(flow);
    }

    function finish(flow){
        var branchName = gitService.reference();
        flow
            .currentBranch()
            .then(function(result, control){
                branchName.value = result.data.gitCurrentBranch.name;
                $scope.$root.rootScopeCurrentBranch = branchName.value;
                messageService.info("We have updated your branch from its remote branch and from remote master.");
                validate();
                control.accept();
            }).then(function(result, control){
               loggingService.info('cms-global-controllerJs',"We have finished updating branch " + branchName.value);
               if($scope.requestedLocationPath.indexOf("conflict-list") == -1) {
                $location.path($scope.requestedLocationPath);
               } else {
                   $location.path("/menu");
               }

               control.accept();
            });
    }

    function validate(){
        $http.get('/cms/validate',{params : {force:false}}).then(function(response){
            $scope.validation = response.data.validation
        }, errorService.errorHandler);
    }
}]);