cmsApp.controller('JsonEditorController', [
    '$scope',
    'jsonEditorService',
    function ($scope, jsonEditorService) {
        $scope.defaultPattern=/.*/;
        var eventCallbacks = [];

        $scope.$emit("registerJsonEditorCallbackEvent", function (callback) {
            eventCallbacks.push(callback);
        });


        $scope.uiTypeRenderers = {
            "text-disabled": "text-disabled.html",
            "textarea": "text-area.html",
            "image-url": "image-url.html",
            "background-image-url": "background-image-url.html",
            "url": "url.html",
            "enum": "enum.html",
            "multi-select": "multi-select.html",
            "text": "default-form-control.html"
        };

        $scope.addItem = function (array) {
            jsonEditorService.addItem(array);
            $scope.moduleForm.$dirty = true;

            angular.forEach(eventCallbacks, function (callback) {
                callback.addItem(array);
            });
        };

        $scope.removeItem = function (array, index) {
            jsonEditorService.removeItem(array, index);
            $scope.moduleForm.$dirty = true;

            angular.forEach(eventCallbacks, function (callback) {
                callback.removeItem(array);
            });
        };

        $scope.moveUp = function (array, index) {
            jsonEditorService.moveUp(array, index);
            $scope.moduleForm.$dirty = true;

            angular.forEach(eventCallbacks, function (callback) {
                callback.moveUp(array);
            });
        };

        $scope.moveDown = function (array, index) {
            jsonEditorService.moveDown(array, index);
            $scope.moduleForm.$dirty = true;

            angular.forEach(eventCallbacks, function (callback) {
                callback.moveDown(array);
            });
        };

        $scope.browseServer = function (modelObj, index) {

            var startupPath = trimImgPathFromUrl(modelObj.value);
            var finder = new CKFinder();
            finder.resourceType = 'Images';
            finder.basePath = '/CKFinderJava/ckfinder/ckfinder.html?type=Images';
            finder.startupPath = startupPath;
            finder.selectActionFunction = function (fileUrl, dataObj) {
                modelObj.value = fileUrl;
                $scope.moduleForm.$dirty = true;
                $scope.$apply();
            };

            finder.popup();

            function trimImgPathFromUrl(url) {
                var startupPath = 'Images:/www/';
                if (modelObj.value) {
                    var regex = /.*www\/(.*)\/.*/;
                    var imagePath = modelObj.value.replace(regex, "$1");
                    startupPath += imagePath + "/";
                } else {
                    startupPath += '_assets/img/';
                }
                return startupPath;
            }
        };

        $scope.nextRenderer = function (decorators, currentRenderer) {
            if (decorators) {
                if (currentRenderer) {
                    var idx = decorators.indexOf(currentRenderer);
                    if (idx == -1 || idx == decorators.length - 1) {
                        return "leaf_node_renderer.html";
                    }
                    return decorators[idx + 1];
                } else {
                    return decorators[0];
                }
            } else {
                return "leaf_node_renderer.html";
            }
        };
        $scope.getKeys = function (obj) {
            if (!obj) {
                return [];
            }

            var keys = Object.keys(obj);
            var hashKeyIndex = keys.indexOf("$$hashKey");
            if (hashKeyIndex != -1) {
                keys.splice(hashKeyIndex, 1);
            }
            return keys;
        };

        $scope.clearValue = function (modelObj) {
            jsonEditorService.clearValue(modelObj);
            $scope.moduleForm.$dirty = true;
        };

        $scope.isEmpty = function (leaf) {
            return !angular.isDefined(leaf.value) || leaf.value == null || leaf.value == "";
        }
    }]);