cmsApp.controller('MergeBranchController', [
    '$scope',
    '$http',
    '$window',
    '$rootScope',
    '$cookies',
    '$q',
    '$location',
    'userService',
    'errorService',
    'messageService',
    'gitService',
    'loggingService',
    'conflictService',
    'editorCommonService',
    'commonUtilService',
    'dateService',
    'dbService',
function ($scope,
          $http,
          $window,
          $rootScope,
          $cookies,
          $q,
          $location,
          userService,
          errorService,
          messageService,
          gitService,
          loggingService,
          conflictService,
          editorCommonService,
          commonUtilService,
          dateService,
          dbService) {


    $scope.$parent.pageTitle = "Merge Branch";

    $window._testSupport = {
        isDocReady : function() {
            var phase = $rootScope.$root.$$phase;
            return $http.pendingRequests.length == 0 && phase != '$apply' && phase != '$digest';
        }
    };

    if (!userService.getUserData().hasRole('QA')) {
        messageService.error("You are not authorised to perform Merge Branch.");
        $location.path("/");
    }

    var branchName =  gitService.reference();

    gitService
        .setErrorMessage("Loading merge branch page failed.")
        .pruneDeletedBranches()
        .then(function(result, control){
            if (result.data.gitPruneDeletedBranches.deletedCurrentBranch){
                messageService.error("The branch you were on has been merged by another user. You are now on the master branch. Please select or create another branch.");

                $location.path('/');
                $scope.$emit('refreshGlobalCtrl');

                control.reject();
            }
            control.accept();
        })
        .currentBranch()
        .then(function (result, control) {
            if(result.data.gitCurrentBranch) {
                branchName.value = result.data.gitCurrentBranch.name;
            }
            control.accept();
        })
        .then(function (result, control) {
            $http.get('/cms/ajax/db/atomic/branchdetails',  {params: {branch: branchName.value}})
            .then(function(result){
                var branchDetail = result.data.branchDetail;
                $scope.currentBranchDetails = branchDetail;
                $scope.changedFiles = branchDetail.changedFiles;
                control.accept();
            }, function(error) {
                control.reject();
                errorService.errorHandler(error)
            });
        });

    $scope.confirmMerge = function (user) {
        angular.element("#mergeConfirmationModal").modal({
            show:true
        })
    };

    $scope.merge = function(){

        var branchName = gitService.reference();

        var flow = gitService
            .setErrorMessage("Merging failed.")
            .currentBranch()
            .then(function(result, control){
                branchName.value = result.data.gitCurrentBranch.name;

                dbService.publishState().then(function(result){
                    var publishState = result.data.publishState;
                   if(_.isEqual(publishState.status, 'PUBLISHING')) {
                        messageService.error("You cannot merge this branch, as publish is currently in progress."
                           + " Publish Started By : " +publishState.user +
                           " Started : "+dateService.getDiffInMinAndSec(publishState.startTime.millis,publishState.currentTime.millis) +" ago");
                        loggingService.info("cms-merge-branch-controllerJS", "merging canceled due to in progress publish to live operation ");
                        control.reject();
                } else {
                    control.accept();
                }})
            })
            .status()
            .then(function(result, control){
                if(!result.data.gitStatus.clean) {
                    messageService.error("You cannot merge this branch, as you have pending changes. Please save all changes before merging.");
                    loggingService.info("cms-merge-branch-controllerJS", "merging canceled due to dirty workspace");
                    control.reject();
                } else {
                    control.accept();
                }
            })
            .fetch()
            .merge('remotes/origin/master')
            .then(function(result, control){
                if (result.data.gitConflict){

                    conflictService.handleConflict(result.data.gitConflict,
                        "Conflict while merging changes from production.",
                        branchName.value,
                        "remotes/origin/master",
                        function(){
                            finish(gitService.flow().setErrorMessage("Continuing merge, after conflict with remote master failed."));
                        });

                    control.reject();
                } else {
                    control.accept();
                }
            });

        finish(flow);
    };

     $scope.editModule = function (filePath) {
        editorCommonService.editModule('/'+filePath);
    };

    $scope.canPreview= function (filePath) {
        return commonUtilService.canBePreviewed(filePath);
    };

    function finish(flow) {

        var branchName = gitService.reference();
        var beforeMergeHeadRef = gitService.reference();
        var afterMergeHeadRef = gitService.reference();

        flow.currentBranch()
            .then(function(result, control){
                branchName.value = result.data.gitCurrentBranch.name;
                control.accept();
            })
            .checkoutBranch('master')
            .getHeadRef('/remotes/origin/master')
            .then(function(result, control) {
                if(result.data.gitHeadRef) {
                    beforeMergeHeadRef.value = result.data.gitHeadRef.headRef;
                    control.accept();
                }
            })
            .merge(branchName)
            .then(function(result, control){
                if (result.data.gitConflict){
                    messageService.error("Failed to merge with local master due to conflict.  Please try again.");
                    control.reject();
                } else {
                    control.accept();
                }
            })
            .getHeadRef('master')
            .then(function(result, control) {
                if(result.data.gitHeadRef) {
                    afterMergeHeadRef.value = result.data.gitHeadRef.headRef;
                    control.accept();
                }
            })
            .push()
            .deleteBranch(branchName)
            .then(function(result,control) {
                    $scope.decoratorFragmentFile = 'fragments/consumer-head-background-styles.vm'
                    return control.accept($http.get("/cms/edit/fragment", {params: {fragment: $scope.decoratorFragmentFile}}));
                },errorService.errorHandler)
            .then(function(result,control){
                loggingService.info('cms-merge-branch-controllerJs', 'Loaded decorator fragment ' + $scope.decoratorFragmentFile );
                var decoratorFragmentContents = result.data.getFragment.content;
                var cssTimeStampPattern = /#set\(\$assetTimestamp[\s]*=[\s]*'([0-9]*)'\)/i
                decoratorFragmentContents = commonUtilService.replaceMatchingContent(decoratorFragmentContents,cssTimeStampPattern,dateService.getCurrentTimeStamp());
                return control.accept($http.post("/cms/edit/fragment", {content: decoratorFragmentContents, fragmentPath: $scope.decoratorFragmentFile}));
            },errorService.errorHandler)
            .then(function(result,control){
                loggingService.info('cms-merge-branch-controllerJs', 'Saved decorator fragment with updated timestamp ' + $scope.decoratorFragmentFile);
                return control.accept($http.get('/cms/ajax/git/atomic/add', {params: {file: $scope.decoratorFragmentFile}}));
            },errorService.errorHandler)
            .then(function(result,control){
                loggingService.info('cms-merge-branch-controllerJs', 'Added decorator fragment to git ' + $scope.decoratorFragmentFile);
                return control.accept($http.get('/cms/ajax/git/atomic/commit', {params: {message: "commited decorator fragment with updated timestamp" }}));
            },errorService.errorHandler)
            .then(function(result,control){
                loggingService.info('cms-merge-branch-controllerJs', 'commited decorator fragment to master branch ' + $scope.decoratorFragmentFile);
                return control.accept($http.get('/cms/ajax/git/atomic/push'));
            },errorService.errorHandler)
            .then(function(result, control){
                control.accept($http.post('/cms/ajax/db/atomic/merge',{
                                                          userName: $rootScope.rootScopeCurrentUser.user,
                                                          branch: branchName.value,
                                                          beforeMergeHeadRef: beforeMergeHeadRef.value,
                                                          afterMergeHeadRef: afterMergeHeadRef.value }));
            })
            .then(function(result, control){
                $rootScope.rootScopeCurrentBranch = "master";

                messageService.info("Successfully committed on  " + new Date().toString());
                loggingService.info("cms-merge-branch-controllerJS", "Merged branch " + branchName.value + ". This branch has been deleted.");
                $location.path("/");
                control.accept();
            });
    }
}]);