"use strict";
cmsApp.controller('PushToLiveController',
    function ($scope, $http, $window, $rootScope,$location, $timeout,$filter, gitService, userService, messageService,
    dbService, errorService, editorCommonService, loggingService, rsyncService, dateService,commonUtilService) {
        var headRef;
        $scope.$parent.pageTitle = "Publish To Live";
        $scope.changedFiles = [];
        $scope.showMergedBranches = false;
        $scope.showChangedFiles = false;
        $scope.currentTab = "pushToLiveTab";

        $window._testSupport = {
            isDocReady: function () {
                var phase = $rootScope.$root.$$phase;
                return $http.pendingRequests.length == 0 && phase != '$apply' && phase != '$digest';
            }
        };

        if (!userService.getUserData().hasRole('PUBLISHER')) {
            messageService.error("You are not authorised to perform Publish to Live.");
            $location.path("/");
        }

        $scope.getPublishingState = function() {
            loggingService.info("cms-push-to-live-controllerJS","getting current publishing state");
            $http.get('/cms/sysadmin/publishingstate').then(function(response) {
                $scope.publishState = response.data.publishState;
                loggingService.info("cms-push-to-live-controllerJS","current publishing state is " +$scope.publishState.status);
                if(_.isEqual($scope.publishState.status, 'READY')) {
                    $http.get('/cms/ajax/configuration/pushtolive').then(function(response) {
                        $scope.pushToLiveUsingRSync= response.data.configuration.configMap[CMS_PUSH_TO_LIVE_RSYNC]
                        loggingService.info("cms-push-to-live-controllerJS","push to live using rsync is configured as " +$scope.pushToLiveUsingRSync);
                        $scope.getRecentPushToLiveRecord();
                    },errorService.errorHandler);
                }  else if(_.isEqual($scope.publishState.status, 'PUBLISHING')) {
                    $scope.enablePublishProgress()
                }
            },errorService.errorHandler)
        };

        $scope.getPublishingState();

        $scope.getDuration=function (startDateTime,endDateTime) {
            var startDate = new Date(startDateTime);
            var endDate = new Date(endDateTime);
            if ($scope.publishState) {
             var duration =  dateService.getDiffInMinAndSec(startDate,endDate);
                 if(duration == -1) {
                         return "Unknown";
                 } else {
                     return duration;
                 }
            }
            return "Unknown";
        }

        $scope.timeDiff = function(startTime, endTime) {
            var duration = dateService.getDiffInMinAndSec(startTime, endTime);
            if(duration == -1) {
                return "Unknown";
            } else {
                return duration;
            }
        }


        $scope.pollPublishingState = function(){
            $rootScope.spinerEnabled = false;
            if($scope.$parent) {
             $http.get('/cms/sysadmin/publishingstate').then(function(response) {
                $scope.publishState = response.data.publishState;

                if(_.isEqual($scope.publishState.status, 'PUBLISHING')) {
                   $timeout($scope.pollPublishingState,1000);
                } else {
                    if(_.isEqual($scope.publishState.status, 'READY')) {
                         $http.get('/cms/ajax/configuration/pushtolive').then(function(response) {
                            $scope.pushToLiveUsingRSync= response.data.configuration.configMap[CMS_PUSH_TO_LIVE_RSYNC]
                            $scope.getRecentPushToLiveRecord();
                         },errorService.errorHandler);
                    }
                }
             },errorService.errorHandler);
            }
        };

        $scope.getRecentPushToLiveRecord = function () {
            $scope.getPage(1);
            dbService.mostRecentPushToLive(function (result) {
                if (result.data.cmsPushToLive) {
                    $scope.mostRecentPushToLiveDate = result.data.cmsPushToLive.pushToLiveDate.millis;
                    $scope.mostRecentRef = result.data.cmsPushToLive.headRef;
                    loggingService.info("cms-push-to-live-controllerJS","most recent pushToLiveDate is " + $filter('date')($scope.mostRecentPushToLiveDate, "MMM d, y h:mm a"));
                    loggingService.info("cms-push-to-live-controllerJS","mostRecentRef is " +$scope.mostRecentRef);
                    dbService.mergedBranchesSinceLastPushToLive($scope.mostRecentPushToLiveDate, function (result) {
                        $scope.mergedBranchesSinceLastPushToLive = result.data.mergedBranches;
                        loggingService.info("cms-push-to-live-controllerJS","merged branches since last pushToLive [ " +getBranchesList($scope.mergedBranchesSinceLastPushToLive) +"]");
                        if (!_.isEmpty($scope.mergedBranchesSinceLastPushToLive)) {
                            $scope.showMergedBranches = true;
                        }
                        $scope.findChangedFilesBetweenCmsPublishes($scope.mergedBranchesSinceLastPushToLive);
                        if (!_.isEmpty($scope.changedFiles)) {
                            $scope.showChangedFiles = true;
                        }
                    });
                }
            });
        };

        $scope.preview = function (fileName) {
            editorCommonService.preview("/"+fileName);
        };

        $scope.confirmPublish = function (user) {
            angular.element("#publishConfirmModal").modal({
                show:true
            })
        };

        $scope.pushToLive = function () {
            /*If a user is already on the pushToLive and some other user initiated PushToLive. Hence additional check
             to avoid multiple pushToLive at same time.*/
            $http.get('/cms/sysadmin/publishingstate').then(function(response) {
                $scope.publishState = response.data.publishState;
                loggingService.info("cms-push-to-live-controllerJS","current publishing state is " +$scope.publishState.status);
                if(_.isEqual($scope.publishState.status, 'READY')) {
                    $scope.enablePublishProgress()
                    $scope.publishInProgress=true;
                    $http.post('/cms/ajax/pushtolive',{timeout: 900000}).then(function(response){
                        if (response && response.data.pushToLive) {
                            loggingService.info("cms-push-to-live-controllerJS","marked publishing state of application to READY");
                            $scope.mostRecentPushToLiveDate = response.data.pushToLive.pushToLiveDate.millis;
                            messageService.info("Successfully published to live.");
                            $scope.mergedBranchesSinceLastPushToLive = {};
                            $scope.showMergedBranches = false;
                            $scope.changedFiles = {};
                            $scope.showChangedFiles = false;
                            $scope.publishInProgress=false;
                        }
                    }, handlePublishToLiveError);
                } else if (_.isEqual($scope.publishState.status, 'PUBLISHING')) {
                    angular.element("#publishInProgressModal").modal({
                        show:true
                    });
                    $scope.mergedBranchesSinceLastPushToLive = {};
                    $scope.changedFiles = {};
                    $scope.publishInProgress=true;
                    $scope.showMergedBranches = false;
                    $scope.showChangedFiles = false;
                }
            });
        };

        function handlePublishToLiveError(errorResult) {

        if (errorResult) {
         errorService.errorHandler(errorResult,"Failed to perform push to live operation");
         messageService.error("Failed to perform push to live operation.");
        } else {
            messageService.error("The publish to live operation is taking longer than usual. " +
                "This may be due to current network speeds, in which case the publish will finish in a few minutes. " +
                "But there may be a problem. Please continue to wait, or revisit this page in 20 minutes. " +
                "If the publish to live operation is still running, please report the issue to the support team.");
        }

        }

        $scope.isFileValidForPreview = function(filePath) {
            if(commonUtilService.isCmsEditable(filePath) || filePath.indexOf(".jpg") != -1 || filePath.indexOf(".jpeg") != -1 || filePath.indexOf(".gif") != -1) {
                return true;
            } else {
                return false;
            }
        };

        $scope.enablePublishProgress = function() {
            $scope.removeSpinner();
            $timeout($scope.pollPublishingState,1000);
        };

        $scope.findChangedFilesBetweenCmsPublishes = function (mergedBranches) {
            var since = $scope.mostRecentRef;
            var until;
            angular.forEach(
                mergedBranches, function (mergedBranch) {
                    until = mergedBranch.beforeMergeHeadRef;
                    getChangedFiles(since, until);
                    since = mergedBranch.afterMergeHeadRef;
                });

             if(_.isEqual($scope.pushToLiveUsingRSync, 'true')) {
                gitService.getHeadRef('master').then(function (result) {
                    until = result.data.gitHeadRef.headRef;
                    getChangedFiles(since, until);
                });
             } else {
                gitService.getRemoteHeadRef().then(function (result) {
                    until = result.data.gitHeadRef.headRef;
                    getChangedFiles(since, until);
                });
             }
        };

        $scope.addSpinner = function () {
            if ($http.defaults.transformRequest.indexOf(spinnerFunction) != -1) {
                $http.defaults.transformRequest.push(spinnerFunction);
            }
        };

        $scope.removeSpinner = function() {
            var n = $http.defaults.transformRequest.length;
            for(var i =0 ; i<n ; i++) {
                if($http.defaults.transformRequest[i] == spinnerFunction) {
                    $http.defaults.transformRequest.splice(i, 1);
                }
            }
        };

        $scope.$on('$destroy', function cleanup() {
            $scope.addSpinner();
        });

        $scope.setCurrentTab = function(tabName) {
            $scope.currentTab = tabName;
        };

        $scope.getActiveClass = function (selectedTab) {
            var tab = selectedTab;
            if(!tab){
                tab = $scope.currentTab
            }
            if((tab == $scope.currentTab)) {
                return 'active in';
            }
        };

        $scope.getPage = function(pageNumber){
            dbService.pushToLiveHistory(pageNumber, 20 , function(result){
                $scope.history = {};
                $scope.history.historyList = result.data.cmsPushToLiveList;
                $scope.history.currentPage = pageNumber;
                $scope.history.totalPages = result.data.totalPages;
            }, errorService.errorHandler);
        };

        function getBranchesList(branchesList) {
            var tmpList = [];
            if (_.isEmpty(branchesList)) {
                tmpList.push("No branches. Some external changes.");
            } else {
                angular.forEach(branchesList, function (branch) {
                   tmpList.push(branch.branchName);
                });
            }

            return tmpList;
        }

        function getChangedFiles(since, until) {
            if (!_.isEqual(since, until)) {
                gitService.getChangedFiles(since, until, errorService.errorHandler)
                    .then(function (result, control) {
                        if (result.data.gitLog) {
                            var gitChangedFiles = result.data.gitLog.changedFiles;
                            $scope.changedFiles = _.uniq(_.union($scope.changedFiles, gitChangedFiles), function (changedfile) {
                                return changedfile.fileName;
                            });
                            if (!_.isEmpty($scope.changedFiles)) {
                                $scope.showChangedFiles = true;
                            }
                            control.accept();
                        }
                    });
            }
        }
    }
);
