cmsApp.controller('UpdateAndResetWorkSpaceController', [
    '$scope',
    '$window',
    '$http',
    '$rootScope',
    'errorService',
    'messageService',
    'gitService',
    'loggingService',
    '$location',
    'commonUtilService',
    'conflictService',
    'scssService',
function UpdateAndResetWorkSpaceController($scope,
                               $window,
                               $http,
                               $rootScope,
                               errorService,
                               messageService,
                               gitService,
                               loggingService,
                               $location,
                               commonUtilService,
                               conflictService,
                               scssService) {

    $scope.$parent.pageTitle = "Update and Reset Workspace";

    $window._testSupport = {
        isDocReady : function() {
            var phase = $rootScope.$root.$$phase;
            return $http.pendingRequests.length == 0 && phase != '$apply' && phase != '$digest';
        }
    };

    $scope.onUpdate = function(){
        update();
    };

    $scope.onReset = function(){
        reset();
    };


    function reset() {
        var branchName = gitService.reference();
        loggingService.info("cms-update-reset-workspace-controller","initiating reset workspace");
        var flow = gitService.setErrorMessage("Reset workspace failed.")
            .cleanWorkSpace().then(function(result, control) {
                loggingService.info("cms-update-reset-workspace-controller","user workspace is cleaned successfully.");
                control.accept();
            })
            .reset()
            .then(function(result, control) {
                $location.path("/update-reset");
                messageService.info("Successfully reset your workspace with the remote head.");
                loggingService.info("cms-update-reset-workspace-controller","user workspace is reset successfully.");
                control.accept();
            })
    }

    function update() {
        var branchName = gitService.reference();

        var flow = gitService.setErrorMessage("Updating local content from remote failed.")
        .currentBranch()
        .then(function(result, control){
            branchName.value = result.data.gitCurrentBranch.name;
            control.accept();
        })
        .status()
        .then(function(result, control){
            if(!result.data.gitStatus.clean){
                control.reject();
                messageService.error("There are unsaved changes.");
            } else {
                control.accept();
            }
        })
        .pull()
        .then(function(result, control){
            if (result.data.gitConflict){
                loggingService.info("cms-update-reset-workspace-controller","conflicts found while updating workspace from remote.");
                    conflictService.handleConflict(result.data.gitConflict,
                        "Conflict while merging branch changes.",
                        branchName.value,
                        "remotes/origin/" + branchName.value,
                        function(){
                            $location.path("/update-reset");
                            commenceMasterMerge(branchName.value, gitService.flow().setErrorMessage("Failure while continuing update after merge with remote."));
                        });

                    control.reject();
            } else {
                loggingService.info("cms-update-reset-workspace-controller","workspace updated from remote branch successfully.");
                scssService.onPullRegenerateScss(
                                result,
                                function(){
                                    loggingService.info("cms-update-reset-workspace-controller","generated scss successfully.");
                                    control.accept()},
                                function(error){control.reject();errorService.errorHandler(regScssRes.error);});
            }
        });

        commenceMasterMerge(branchName, flow);
    }

    function commenceMasterMerge(branchName, flow){

        flow.merge('remotes/origin/master')
        .then(function(result, control){
             if (result.data.gitConflict){
                 loggingService.info("cms-update-reset-workspace-controller","conflicts found while updating workspace from remote master.");
                 conflictService.handleConflict(result.data.gitConflict,
                     "Conflict while merging changes from production.",
                     branchName,
                     "remotes/origin/master",
                     function(){
                         $location.path("/update-reset");
                         messageService.info("Successfully updated local branch with remote as well as remote master changes.");
                     });

                 control.reject();
             } else {
                 control.accept();
             }
        })
        .then(function(result, control) {
            $location.path("/update-reset");
            loggingService.info("cms-update-reset-workspace-controller","Successfully updated local branch with remote as well as remote master changes.");
            messageService.info("Successfully updated local branch with remote as well as remote master changes.");
            control.accept();
        })
    }
} ]);

