cmsApp.controller('UserDetailsController', [
    '$scope',
    '$http',
    '$window',
    '$rootScope',
    '$location',
    'userService',
    'errorService',
    'messageService',
    'loggingService',
    'userRolesService',
    function UserDetailsController($scope, $http, $window, $rootScope, $location, userService, errorService, messageService, loggingService, userRolesService) {
        $window._testSupport = {
            isDocReady: function () {
                var phase = $rootScope.$root.$$phase;
                return $http.pendingRequests.length == 0 && phase != '$apply' && phase != '$digest';
            }
        };
        $scope.isEditPage = false;

        $scope.$parent.pageTitle = 'Add User';
        if ($rootScope.selectedUser) {
            $scope.$parent.pageTitle = 'Edit User';
            $scope.isEditPage = true;
        }


        if (!userService.getUserData().hasRole('ADMIN')){
            messageService.error("You are not authorised to perform User Administration operations.");
            $location.path("/");
            return;
        }

        $scope.roles = userRolesService.userRoles();

        if ($rootScope.selectedUser != null) {
            $scope.newUser = $rootScope.selectedUser;
            $scope.newUser.suspended = $scope.newUser.suspended.toString();
            userRolesService.selectRoles($scope.roles, $scope.newUser.roles)
            $scope.hidePassword = true;
        } else {
            $scope.newUser = {
                suspended :  'false'
            };
            $scope.hidePassword = false;
        }

        $scope.onSave = function () {
            $scope.newUser.roles =  userRolesService.getSelectedRoles($scope.roles);

            if ($rootScope.selectedUser) {
                updateUser();
            } else {
                addUser();
            }
        };

        $scope.onCancel = function () {
            $rootScope.selectedUser = null;
            $location.path("/user-management");
        };

        $scope.resetConfirmPassword = function() {
            $scope.newUser.password_c = '';
        };

        $scope.canChangeStatus=function (userName) {
            return userName != $rootScope.rootScopeCurrentUser.user;
        }
        function addUser(){
            $http.post('/cms/admin/user', $scope.newUser)
                .then(function (response) {
                    loggingService.info("cms-user-details-controllerJS", "added new user " + userObjForLog($scope.newUser));

                    messageService.info("Added the user successfully.")

                    $location.path("/user-management");
                }, errorService.errorHandler);
        }

        function updateUser(){
            delete $scope.newUser.lastLogInDate;
            $http.put('/cms/admin/user/' + $scope.newUser.userName, $scope.newUser)
                .then(function(response){
                    loggingService.info("cms-user-details-controllerJS","updated user " +  userObjForLog($scope.newUser));
                    $rootScope.selectedUser = null;

                    messageService.info("Updated the user information successfully.")

                    $location.path("/user-management");
                }, errorService.errorHandler);
        }

        function userObjForLog(userObj) {
            var userObjForLog = userObj;
            delete userObjForLog.password;
            delete userObjForLog.password_c;
            delete userObjForLog.$$hashKey;
            return JSON.stringify(userObjForLog);
        }

    }]);
