cmsApp.controller('UserManagementController', [
    '$scope',
    '$http',
    '$window',
    '$rootScope',
    '$location',
    'userService',
    'errorService',
    'messageService',
    'loggingService',
    'userRolesService',
    function UserManagementController($scope, $http, $window, $rootScope, $location, userService, errorService, messageService, loggingService, userRolesService) {
        $scope.$parent.pageTitle = "User Administration";
        $window._testSupport = {
            isDocReady: function () {
                var phase = $rootScope.$root.$$phase;
                return $http.pendingRequests.length == 0 && phase != '$apply' && phase != '$digest';
            }
        };

        if (!userService.getUserData().hasRole('ADMIN')) {
            messageService.error("You are not authorised to perform User Administration operations.");
            $location.path("/");
            return;
        }

        var displayAllUsers = userService.getUserData().hasRole("GOD");

        $http.get("/cms/admin/user")
            .then(function (result) {
                $scope.users = _(result.data.cmsUsers.users).filter(function(user) { return displayAllUsers || user.roles.indexOf("GOD") == -1;  })

            }, errorService.errorHandler);


        $scope.onAdd = function () {
            $rootScope.selectedUser = null;
            $location.path("/user-details");
        };

        $scope.onEdit = function (user) {
            $rootScope.selectedUser = user;
            $location.path("/user-details");
        };

        $scope.showDeleteConfirmationDialog = function (user) {
            $scope.currentUser = user;
            angular.element("#deleteConfirmationModal").modal({
                show: true
            })
        }

        $scope.onRemove = function (user) {
            $http.delete('/cms/admin/user/' + user.userName)
                .then(function (data) {
                    loggingService.info("cms-user-management-controllerJS", "user deleted " + user.userName);
                    $scope.users.splice($scope.users.indexOf(user), 1);
                    messageService.info("Successfully deleted " + user.userName + ".")
                }, errorService.errorHandler);
        };

        $scope.onChangePassword = function (user) {
            $rootScope.selectedUser = user;
            $location.path("/change-password");
        };

        $scope.getRoleDesc = function (role) {
            return userRolesService.userRoles()[role].label;
        }
    }]);

