cmsApp.controller('ViewChangesController', [
    '$scope',
    '$http',
    '$window',
    '$rootScope',
    'errorService',
    'messageService',
    'gitService',
    'dbService',
    'fsService',
    'loggingService',
    'scssService',
    '$location',
    'commonUtilService',
    'conflictService',
    'editorCommonService',
    'scssService',
function ViewChangesController($scope,
                               $http,
                               $window,
                               $rootScope,
                               errorService,
                               messageService,
                               gitService,
                               dbService,
                               fsService,
                               loggingService,
                               scssService,
                               $location,
                               commonUtilService,
                               conflictService,
                               editorCommonService,
                               scssService) {

    $scope.$parent.pageTitle = "Pending Changes";

    $window._testSupport = {
        isDocReady : function() {
            var phase = $rootScope.$root.$$phase;
            return $http.pendingRequests.length == 0 && phase != '$apply' && phase != '$digest';
        }
    };

    $scope.onCommit = function(form){
        commit(form);
    };

    $scope.onRevert = function () {
        revert();
    };

    $scope.onUpdate = function(){
        update();
    };

    $scope.onFormChange = function(){
        $scope.disableOperations = !isAnyFileSelected();
    };

    $scope.editModule = function (filePath) {
        editorCommonService.editModule('/' + filePath);
    };

    $scope.isEditable = function (file) {
        return commonUtilService.isCmsEditable(file);
    };

    refreshStatus(gitService.flow().setErrorMessage("Refreshing status of local workspace failed."));

    function isAnyFileSelected(){
        var result = false;

        function isFileSelected(files, onlyControlFiles){
            angular.forEach(files, function(file) {
                if(file.selected){
                    result = true;
                }
            });
        }

        isFileSelected($scope.changesToCommit.newPages, true);

        isFileSelected($scope.changesToCommit.changedPages, false);

        isFileSelected($scope.changesToCommit.removedPages, true);

        return result;
    }

    function isControlFile(file){
        return commonUtilService.isControlFile(file);
    }

    function isScss(file){
        return commonUtilService.isScssFile(file);
    }

    function sortIntoFileSets(files, controlFilesOnly){
        var fileSets = {};
        this.controlFilesOnly = controlFilesOnly;

        //find all keys for sets
        angular.forEach(files, function(item) {
            if (this.controlFilesOnly && isControlFile(item.path)){
                var key = item.path.substring(0,item.path.lastIndexOf('/'));
                fileSets[key] = {files: [], selected: item.selected};
            } else if(!this.controlFilesOnly && !isScss(item.path)) {
                var indexOfLastSlash = item.path.lastIndexOf('/');
                var indexFileNameWithoutExt = item.path.indexOf(".", indexOfLastSlash);

                var key = item.path.substring(0, indexFileNameWithoutExt);
                fileSets[key] = {files: [], selected: item.selected};
            }
        });

        //add each file to the file sets
        angular.forEach(files, function(item) {
            var bPushedFileToSet = false;
            angular.forEach(fileSets, function(fileSet, key){
                var filePath = item.path;
                if (filePath.lastIndexOf("scss") == filePath.length - 4){
                    filePath = filePath.replace("_assets/scss/pages/","");
                }
                if (this.controlFilesOnly && filePath.indexOf(key+"/") == 0){
                    fileSet.files.push(item.path);
                    bPushedFileToSet = true;
                }
                if (!this.controlFilesOnly && filePath.indexOf(key) == 0){
                    fileSet.files.push(item.path);
                    bPushedFileToSet = true;
                }
            })

            if(!bPushedFileToSet) {
                var fileSet = {files: [], selected: item.selected};
                fileSet.files.push(item.path);
                fileSets[item.path] = fileSet;
            }
        });
        return fileSets;
    }

    function refreshStatus(flow) {
        function translate(lhs, rhs){
            for (var index in lhs) {
                var path = lhs[index];
                //we don't want the CMS system to see any css files. They are generated by grunt.
                if (path.lastIndexOf(".css") != path.length - 4) {
                    rhs.push({selected: true, path: lhs[index]})
                }
            }
        }

        flow.then(function(result, control){
            $scope.changesToCommit = {
                newPages: [],
                changedPages: [],
                removedPages: []
            };
            control.accept();
        })
        .status()
        .then(function(result, control){
            translate(result.data.gitStatus.untracked, $scope.changesToCommit.newPages);
            translate(result.data.gitStatus.modified, $scope.changesToCommit.changedPages);
            translate(result.data.gitStatus.removed, $scope.changesToCommit.removedPages);
            translate(result.data.gitStatus.missing, $scope.changesToCommit.removedPages);

            //group all the files
            $scope.changesToCommit.newPages = sortIntoFileSets($scope.changesToCommit.newPages, true);
            $scope.changesToCommit.changedPages = sortIntoFileSets($scope.changesToCommit.changedPages, false);
            $scope.changesToCommit.removedPages = sortIntoFileSets($scope.changesToCommit.removedPages, true);

            if(result.data.gitStatus.conflicting.length > 0)
                messageService.error("Merge conflict : "+result.data.gitStatus.conflicting);
            $scope.onFormChange();
            control.accept();
        });
    }

    function commit(form) {

        if (!form.$valid) {
            return;
        }

        var flow = gitService.flow().setErrorMessage("Committing changes failed.");
        var changedFiles = gitService.reference();
        changedFiles.value = [];


        //go through newPages sets and add files if index file is selected
        angular.forEach($scope.changesToCommit.newPages, function(fileSet, key){
            if (fileSet.selected){
                angular.forEach(fileSet.files, function(file){
                    loggingService.info("cms-view-changes-controllerJS","committing file " + file);
                    flow.add(file);
                    changedFiles.value.push({fileName: file, action: 'ADD'});
                });
            }
        });

        //go through modifiedPages set and add files if they are selected
        angular.forEach($scope.changesToCommit.changedPages, function(fileSet, key) {
            if(fileSet.selected) {
                angular.forEach(fileSet.files, function(file){
                    loggingService.info("cms-view-changes-controllerJS","committing file " + file);
                    flow.add(file);
                    changedFiles.value.push({fileName: file, action: 'MODIFY'});
                });
            }
        });

        //go through removedPages sets and delete files if index file is selected
        angular.forEach($scope.changesToCommit.removedPages, function(fileSet, key){
            if (fileSet.selected){
                angular.forEach(fileSet.files, function(file){
                    loggingService.info("cms-view-changes-controllerJS","deleting file " + file);
                    flow.remove(file);
                    changedFiles.value.push({fileName: file, action: 'DELETE'});
                });
            }
        });

        var branchName = gitService.reference();

        flow.commit($scope.changesToCommit.message)
            .currentBranch()
            .then(function(result, control){
                branchName.value = result.data.gitCurrentBranch.name;

                dbService.commit(branchName.value, changedFiles.value, function (result) {
                    dbService.addContributor(branchName.value, $rootScope.rootScopeCurrentUser.user, function(result) {
                        control.accept();
                    }, function(error) {
                        control.reject();
                        errorService.errorHandler(error);
                    });
                }, function(error) {
                    control.reject();
                    errorService.errorHandler(error);
                });
            })
            .pull()
            .then(function(result, control){
                if (result.data.gitConflict){

                    conflictService.handleConflict(result.data.gitConflict,
                        "Conflict while merging branch changes.",
                        branchName.value,
                        "remotes/origin/" + branchName.value,
                        function(){
                            gitService
                                .setErrorMessage("Continuing after merge conflict while committing failed.")
                                .push()
                                .then(function(result, control){
                                    $location.path("/view-changes");
                                    messageService.info("Operation successful.");
                                });
                        });

                    control.reject();
                } else {
                    scssService.onPullRegenerateScss(
                                result,
                                function(){control.accept()},
                                function(error){control.reject();errorService.errorHandler(regScssRes.error);});
                }
            })
            .push();

        refreshStatus(flow);

        flow.then(function(result, control){
                messageService.info("Operation successful.");
                loggingService.info("cms-view-changes-controllerJS", "committed changes");
                control.accept();
            });
    }

    function collectScssFromChangedFiles(){
        var scssFiles = [];
        angular.forEach($scope.changesToCommit,function(filesUnderGitStatus, key) {
             angular.forEach(filesUnderGitStatus, function(fileSet, key) {
                if(fileSet.selected) {
                   angular.forEach(fileSet.files, function(file) {
                        if(isScss(file)) { scssFiles.push(file); }
                   });
                }
             });
        });
        return scssFiles;
    }

    function allContexts(scssFiles){
        var allContexts = [];
        angular.forEach(scssFiles,function(file){
            if(file.indexOf("/")==0){
                file = file.substring(1);
            }
            var context = file.split("/")[0]
            if (allContexts.indexOf(context) == -1 ) {
                allContexts.push(context);
            }
        });
        return allContexts;
    }

    function revert() {
        var flow = gitService.flow().setErrorMessage("Reverting local changes failed.");
        //go through newPages sets and delete files if index file is selected
        angular.forEach($scope.changesToCommit.newPages, function(fileSet, key){
            if (fileSet.selected){
                angular.forEach(fileSet.files, function(file){
                    loggingService.info("cms-view-changes-controllerJS","reverting file " + file);
                    flow.then(function(result, control){
                        fsService.deleteFile(file, function(result){
                            control.accept();
                        }, function(error) {
                            control.reject();
                            errorService.errorHandler(error);
                        });
                    });

                });
            }
        });

        angular.forEach($scope.changesToCommit.changedPages, function(fileSet, key) {
            if (fileSet.selected){
                angular.forEach(fileSet.files, function(file){
                    loggingService.info("cms-view-changes-controllerJS","reverting file " + file);
                    flow.checkoutFile(file);
                });
            }
        });

        //go through removedPages sets and revert files if index file is selected
        angular.forEach($scope.changesToCommit.removedPages, function(fileSet, key){
            if (fileSet.selected){
                angular.forEach(fileSet.files, function(file){
                    loggingService.info("cms-view-changes-controllerJS","reverting file " + file);
                    flow.checkoutFile(file);
                });
            }
        });

        flow.then(function(result,control) {
            var scssFiles = collectScssFromChangedFiles();
            if(scssFiles.length > 0) {
                scssService.runGrunt(
                    'www',
                    function(gruntResult){
                        control.accept();
                    },
                    function(error) {
                         control.reject();
                         errorService.errorHandler(error);
                    });
            } else {
                control.accept();
            }
        });

        refreshStatus(flow);

        flow.then(function(result, control){
                loggingService.info("cms-view-changes-controllerJS", "revert complete");
                messageService.info("Operation successful.");
                control.accept();
            });
    }


    function update() {
        var branchName = gitService.reference();

        var flow = gitService.setErrorMessage("Updating local content from remote failed.")
        .currentBranch()
        .then(function(result, control){
            branchName.value = result.data.gitCurrentBranch.name;
            control.accept();
        })
        .status()
        .then(function(result, control){
            if(!result.data.gitStatus.clean){
                control.reject();
                messageService.error("There are unsaved changes.");
            } else {
                control.accept();
            }
        })
        .pull()
        .then(function(result, control){
            if (result.data.gitConflict){
                loggingService.info("cms-view-changes-controller","conflicts found while updating workspace from remote.");
                    conflictService.handleConflict(result.data.gitConflict,
                        "Conflict while merging branch changes.",
                        branchName.value,
                        "remotes/origin/" + branchName.value,
                        function(){
                            $location.path("/view-changes");
                            commenceMasterMerge(branchName.value, gitService.flow().setErrorMessage("Failure while continuing update after merge with remote branch."));
                        });

                    control.reject();
            } else {
                loggingService.info("cms-view-changes-controller","workspace updated from remote branch successfully.");
                scssService.onPullRegenerateScss(
                                result,
                                function(){control.accept()},
                                function(error){control.reject();errorService.errorHandler(regScssRes.error);});
            }
        });

        commenceMasterMerge(branchName, flow);
    }

    function commenceMasterMerge(branchName, flow){

        flow.merge('remotes/origin/master')
        .then(function(result, control){
             if (result.data.gitConflict){
                 loggingService.info("cms-view-changes-controller","conflicts found while updating workspace from remote master.");
                 conflictService.handleConflict(result.data.gitConflict,
                     "Conflict while merging changes from production.",
                     branchName,
                     "remotes/origin/master",
                     function(){
                         $location.path("/view-changes");
                         messageService.info("Successfully updated local branch with remote as well as remote master changes.");
                     });

                 control.reject();
             } else {
                 control.accept();
             }
        })
        .then(function(result, control) {
            $location.path("/view-changes");
            loggingService.info("cms-view-changes-controller","Successfully updated local branch with remote as well as remote master changes.");
            messageService.info("Successfully updated local branch with remote as well as remote master changes.");
            control.accept();
        })
    }
} ]);

