
cmsApp.factory("errorService", function(loggingService, $rootScope, $location, $q, messageService){

        return {
                errorHandler: function(result, clientMessage) {
                    if (!result){
                        return $q.reject();
                    }

                    var error = {};
                    if (result.constructor.name === "SyntaxError") {

                        if (clientMessage){
                            loggingService.error("cms-error-serviceJS", "syntax error, probably in a javascript document downloaded from the server.");
                        }
                        error.message = result.message;
                        error.code =  "ERROR_CMS_JSON_SYNTAX_ERROR";
                        error.clientMessage = clientMessage;
                        error.httpCode = 200;
                        error.time = Date.parse(new Date());

                        $rootScope.error = error;
                        $location.path("/error");

                    } else if (result.constructor.name === "TypeError") {

                        if (clientMessage){
                            loggingService.error("cms-error-serviceJS", "type error, probably in a javascript document downloaded from the server.");
                        }
                        error.message = result.message;
                        error.code =  "ERROR_CMS_JSON_TYPE_ERROR";
                        error.clientMessage = clientMessage;
                        error.httpCode = 200;
                        error.time = Date.parse(new Date());

                        $rootScope.error = error;
                        $location.path("/error");
                    } else if (result.headers && result.headers()['content-type'].indexOf("application/json") == -1){
                        if (clientMessage){
                            loggingService.error("cms-error-serviceJS", "content type returned from server is not json.  There is probably another error.");
                        }
                        error.message = result.message;
                        error.code =  "ERROR_CMS_UNEXPECTED_RESPONSE_CONTENT_TYPE";
                        error.clientMessage = clientMessage;
                        error.httpCode = result.status;
                        error.time = Date.parse(new Date());

                        $rootScope.error = error;
                        $location.path("/error");
                    } else if (result.data.error) {

                        //This error handler does not result in presenting the error page.  It raises a flash messages
                        //with the content below on the page the user is currently on.
                        if(result.data.error.code == "ERROR_CMS_GIT_REF_NOT_FOUND") {
                            messageService.error("Unfortunately this branch has been merged by another user, and no " +
                                "longer exists. You are still on a local copy of the branch, so you can make a note " +
                                "of your changes using 'View history' and 'Pending changes', but you will have to go" +
                                " to Branches and change branch or create a new one.");
                            return;
                        }

                        if (clientMessage){
                            loggingService.error("cms-error-serviceJS", clientMessage);
                        }

                        result.data.error.clientMessage = clientMessage;
                        result.data.error.httpCode = result.status;
                        if (result.config) {
                            result.data.error.url = result.config.url;
                        }
                        $rootScope.error = result.data.error;
                        $rootScope.error.time = Date.parse(result.data.error.time);
                        $location.path("/error");
                    } else {

                        error.message = result.status + " returned for url " + result.config.url;
                        error.code =  "ERROR_CMS_NON_STANDARD_ERROR_PACKET";
                        error.detailedMessage = result.data;
                        error.httpCode = result.status;
                        error.time = Date.parse(new Date());

                        $rootScope.error = error;
                        $location.path("/error");
                    }

                    return $q.reject(); //this will cause the next promise in the chain to call its error handler
                },
                error: function(code, message){
                    return {
                        data: {
                            error: {
                                code: code,
                                message: message,
                                time: new Date()
                            }
                        }
                    };
                }
            };
    });