cmsApp.factory("gitService", function ($rootScope, $location, $q, $http, errorService) {

    function Utils() {
    }

    Utils.dereference = function (argument) {
        if (argument instanceof Reference) {
            return argument.value;
        } else {
            return argument;
        }
    };

    Utils.resolveErrorHandler = function (errorCallback, message) {
        var messageHandlingErrorHandler = function (result) {
            return errorService.errorHandler(result, message);
        };

        return errorCallback == undefined ? messageHandlingErrorHandler : errorCallback;
    };

    return {
        startingPromise: function () {
            var deferred = $q.defer();
            deferred.resolve();
            return deferred.promise;
        },
        setErrorMessage: function (errorMessage) {
            return new Flow(this.startingPromise()).setErrorMessage(errorMessage);
        },
        status: function (errorCallback) {
            return new Flow(this.startingPromise()).status(errorCallback);
        },
        pull: function (errorCallback) {
            return new Flow(this.startingPromise()).pull(errorCallback);
        },
        merge: function (branchName, errorCallback) {
            return new Flow(this.startingPromise()).merge(branchName, errorCallback);
        },
        fetch: function (errorCallback) {
            return new Flow(this.startingPromise()).fetch(errorCallback);
        },
        currentBranch: function (errorCallback) {
            return new Flow(this.startingPromise()).currentBranch(errorCallback);
        },
        checkoutBranch: function (branchName, errorCallback) {
            return new Flow(this.startingPromise()).checkoutBranch(branchName, errorCallback);
        },
        push: function (errorCallback) {
            return new Flow(this.startingPromise()).push(errorCallback);
        },
        pushFromRemoteMaster: function (errorCallback) {
            return new Flow(this.startingPromise()).pushFromRemoteMaster(errorCallback);
        },
        deleteBranch: function (branchName, errorCallback) {
            return new Flow(this.startingPromise()).deleteBranch(branchName, errorCallback);
        },
        exists: function (errorCallback) {
            return new Flow(this.startingPromise()).exists(errorCallback);
        },
        clone: function (errorCallback) {
            return new Flow(this.startingPromise()).clone(errorCallback);
        },
        checkoutFile: function (file, errorCallback) {
            return new Flow(this.startingPromise()).checkoutFile(file, errorCallback);
        },
        add: function (file, errorCallback) {
            return new Flow(this.startingPromise()).add(file, errorCallback);
        },
        commit: function (message, errorCallback) {
            return new Flow(this.startingPromise()).commit(message, errorCallback);
        },
        reset: function (message, errorCallback) {
            return new Flow(this.startingPromise()).reset(message, errorCallback);
        },
        resetToHead: function (message, errorCallback) {
            return new Flow(this.startingPromise()).resetToHead(message, errorCallback);
        },
        cleanWorkSpace: function (message, errorCallback) {
            return new Flow(this.startingPromise()).cleanWorkSpace(message, errorCallback);
        },
        log: function (branchName, errorCallback) {
            return new Flow(this.startingPromise()).log(branchName, errorCallback);
        },
        getHeadRef: function (branchName, errorCallback) {
            return new Flow(this.startingPromise()).getHeadRef(branchName, errorCallback);
        },
        getRemoteHeadRef: function (branchName, errorCallback) {
            return new Flow(this.startingPromise()).getRemoteHeadRef(branchName, errorCallback);
        },
        pruneDeletedBranches: function (errorCallback) {
            return new Flow(this.startingPromise()).pruneDeletedBranches(errorCallback);
        },
        getChangedFiles: function (since, until, errorCallback) {
            return new Flow(this.startingPromise()).getChangedFiles(since, until, errorCallback);
        },
        flow: function () {
            return new Flow(this.startingPromise());
        },
        reference: function () {
            return new Reference();
        }
    };

    function Reference() {
        this.value = null;
        return this;
    }


    function Flow(promise) {
        var self = this;
        this.errorMessage = null;
        this.promise = promise;

        this.setErrorMessage = function (errorMessage) {
            self.errorMessage = errorMessage;
            return this;
        };

        this.status = function (errorCallback) {
            this.promise = this.promise.then(function (result) {
                return $http.get('/cms/ajax/git/atomic/status');
            }, Utils.resolveErrorHandler(errorCallback, self.errorMessage));
            return this;
        };

        this.pull = function (errorCallback) {
            this.promise = this.promise.then(function (result) {
                return $http.get('/cms/ajax/git/atomic/pull');
            }, Utils.resolveErrorHandler(errorCallback, self.errorMessage));
            return this;
        };


        this.currentBranch = function (errorCallback) {
            this.promise = this.promise.then(function (result) {
                return $http.get('/cms/ajax/git/atomic/currentbranch');
            }, Utils.resolveErrorHandler(errorCallback, self.errorMessage));
            return this;
        };

        this.fetch = function (errorCallback) {
            this.promise = this.promise.then(function (result) {
                return $http.get('/cms/ajax/git/atomic/fetch');
            }, Utils.resolveErrorHandler(errorCallback, self.errorMessage));
            return this;
        };

        this.merge = function (branchName, errorCallback) {
            this.promise = this.promise.then(function (result) {
                var branch = Utils.dereference(branchName);
                return $http.get('/cms/ajax/git/atomic/merge', {params: {reference: branch}});
            }, Utils.resolveErrorHandler(errorCallback, self.errorMessage));
            return this;
        };

        this.checkoutBranch = function (branchName, errorCallback) {
            this.promise = this.promise.then(function (result) {
                var branch = Utils.dereference(branchName);
                return $http.get('/cms/ajax/git/atomic/checkoutbranch', {params: {branch: branch}});
            }, Utils.resolveErrorHandler(errorCallback, self.errorMessage));
            return this;
        };

        this.push = function (errorCallback) {
            this.promise = this.promise.then(function (result) {
                return $http.get('/cms/ajax/git/atomic/push');
            }, Utils.resolveErrorHandler(errorCallback, self.errorMessage));
            return this;
        };


        this.pushFromRemoteMaster = function (errorCallback) {
            this.promise = this.promise.then(function (result) {
                return $http.get('/cms/ajax/git/atomic/remote/master/push');
            }, Utils.resolveErrorHandler(errorCallback, self.errorMessage));
            return this;
        };


        this.deleteBranch = function (branchName, errorCallback) {
            this.promise = this.promise.then(function (result) {
                var branch = Utils.dereference(branchName);
                return $http.get('/cms/ajax/git/atomic/deletebranch', {params: {branch: branch}});
            }, Utils.resolveErrorHandler(errorCallback, self.errorMessage));
            return this;
        };

        this.exists = function (errorCallback) {
            this.promise = this.promise.then(function (result) {
                return $http.get('/cms/ajax/git/atomic/exists');
            }, Utils.resolveErrorHandler(errorCallback, self.errorMessage));
            return this;
        };

        this.clone = function (errorCallback) {
            this.promise = this.promise.then(function (result) {
                return  $http.get('/cms/ajax/git/atomic/clone',{timeout: 120000});
            }, Utils.resolveErrorHandler(errorCallback, self.errorMessage));
            return this;
        };

        this.checkoutFile = function (file, errorCallback) {
            this.promise = this.promise.then(function (result) {
                var _file = Utils.dereference(file);
                return  $http.get('/cms/ajax/git/atomic/checkoutfile', {params: {file: _file}});
            }, Utils.resolveErrorHandler(errorCallback, self.errorMessage));
            return this;
        };

        this.add = function (file, errorCallback) {
            this.promise = this.promise.then(function (result) {
                var _file = Utils.dereference(file);
                return  $http.get('/cms/ajax/git/atomic/add', {params: {file: _file}});
            }, Utils.resolveErrorHandler(errorCallback, self.errorMessage));
            return this;
        };

        this.commit = function (message, errorCallback) {
            this.promise = this.promise.then(function (result) {
                var _message = Utils.dereference(message);
                return  $http.get('/cms/ajax/git/atomic/commit', {params: {message: _message }});
            }, Utils.resolveErrorHandler(errorCallback, self.errorMessage));
            return this;
        };

        this.reset = function (errorCallback) {
            this.promise = this.promise.then(function (result) {
                return  $http.get('/cms/ajax/git/atomic/reset');
            }, Utils.resolveErrorHandler(errorCallback, self.errorMessage));
            return this;
        };

        this.resetToHead = function (errorCallback) {
            this.promise = this.promise.then(function (result) {
                return  $http.get('/cms/ajax/git/atomic/resettohead');
            }, Utils.resolveErrorHandler(errorCallback, self.errorMessage));
            return this;
        };

        this.cleanWorkSpace = function (errorCallback) {
            this.promise = this.promise.then(function (result) {
                return  $http.get('/cms/ajax/git/atomic/clean');
            }, Utils.resolveErrorHandler(errorCallback, self.errorMessage));
            return this;
        };

        this.log = function (branchName, errorCallback) {
            this.promise = this.promise.then(function (result) {
                var branch = Utils.dereference(branchName);
                return  $http.get('/cms/ajax/git/atomic/log', {params: {branch: branch }});
            }, Utils.resolveErrorHandler(errorCallback, self.errorMessage));
            return this;
        };

        this.remove = function (file, errorCallback) {
            this.promise = this.promise.then(function (result) {
                var _file = Utils.dereference(file);
                return  $http.get('/cms/ajax/git/atomic/remove', {params: {file: _file }});
            }, Utils.resolveErrorHandler(errorCallback, self.errorMessage));
            return this;
        };

        this.getHeadRef = function (branchName, errorCallback) {
            this.promise = this.promise.then(function (result) {
                return  $http.get('/cms/ajax/git/atomic/headref', {params: {branchName: branchName }});
            }, Utils.resolveErrorHandler(errorCallback, self.errorMessage));
            return this;
        };

        this.getRemoteHeadRef = function (errorCallback) {
            this.promise = this.promise.then(function (result) {
                return  $http.get('/cms/ajax/git/atomic/remote/master/headref');
            }, Utils.resolveErrorHandler(errorCallback, self.errorMessage));
            return this;
        };

        this.pruneDeletedBranches = function (errorCallback) {
            this.promise = this.promise.then(function (result) {
                return  $http.get('/cms/ajax/git/atomic/prunedeletedbranches');
            }, Utils.resolveErrorHandler(errorCallback, self.errorMessage));
            return this;
        };

        this.getChangedFiles = function (since, until, errorCallback) {
            this.promise = this.promise.then(function (result) {
                return  $http.get('/cms/ajax/git/atomic/remote/master/changedfiles', {params: {since: since, until: until}});
            }, Utils.resolveErrorHandler(errorCallback, self.errorMessage));
            return this;
        };

        this.then = function (successCallback, errorCallback) {

            this.promise = this.promise.then(function (result) {
                var self = this;

                this.deferred = $q.defer();

                successCallback(result,
                    {
                        reject: function (result) {
                            self.deferred.reject(result);

                        },
                        accept: function (result) {
                            self.deferred.resolve(result);
                        }
                    });

                return deferred.promise;
            }, Utils.resolveErrorHandler(errorCallback, self.errorMessage));
            return this;
        };
    }
});