cmsApp.factory("jsonDiffProcessorService", function(schemaWalker){

    var background = [];
    var leftLeafNodeDecorators = ['leaf-node-bg-color-decorator.html','leaf-node-copy-value-decorator.html'];
    var rightLeafNodeDecorators = ['leaf-node-bg-color-decorator.html'];

    function setBgColorOfTabs(metaLeft, metaRight) {
        if (metaLeft != undefined && metaRight == undefined) {
            metaLeft.meta.colorScheme = ["Red","black"];
        } else if (metaLeft == undefined && metaRight != undefined) {
            metaRight.meta.colorScheme = ["Green","white"];
        }
    }

    function resetColorScheme(obj){
        if(obj && obj.meta && obj.meta.colorScheme)
            delete obj.meta.colorScheme;
    }

    function setBackgroundColor(metaLeft, metaRight) {
        if(background.length == 0) {
            background=[
                ["ForestGreen","white"],
                ["AntiqueWhite","black"],
                ["Aqua","black"],
                ["Aquamarine","black"],
                ["Azure","black"],
                ["Gold","black"],
                ["Bisque","black"],
                ["BlanchedAlmond","black"],
                ["BlueViolet","white"],
                ["BurlyWood","black"],
                ["CadetBlue","black"],
                ["Coral","black"],
                ["CornflowerBlue","black"],
                ["Olive","white"],
                ["Cyan","black"],
                ["DarkGoldenRod","black"],
                ["DarkGray","black"],
                ["DarkKhaki","black"],
                ["DarkOrange","black"],
                ["DarkOrchid","white"],
                ["DarkSalmon","black"],
                ["DarkSeaGreen","black"],
                ["DarkTurquoise","black"],
                ["RosyBrown","black"],
                ["Thistle","black"],
                ["Violet","black"]
                ];

            background.sort( function() { return Math.random() - 0.5 } );
        }

        var color = background.pop();
        metaLeft.colorScheme = color;
        metaRight.colorScheme = color;
    }

    function walk(schema, left, right){
        function onObjectOrArray(schema, left, right){
            setBgColorOfTabs(left, right);

            if(left && left.meta) {
                left.meta.disabled = true;
            }
        }
        schemaWalker.walk(schema, left, right, {
            onAll: function(schema, left, right){
                resetColorScheme(left);
                resetColorScheme(right);
            },
            onPrimitive: function(schema, left, right){
                var leftValue = undefined;
                var leftMeta = {};

                if (left != undefined) {
                    leftValue = left.value;
                    leftMeta = left.meta;
                    left.meta.disabled = true;
                }
                if (right != undefined) {
                    if (leftValue !=  right.value) {
                        leftMeta.rightConflictedObj = right;
                        setBackgroundColor(leftMeta, right.meta);
                        leftMeta.decorators = leftLeafNodeDecorators;
                        right.meta.decorators = rightLeafNodeDecorators;
                    } else {
                        //we have been specifically requested to leave all fields editable when presenting module diffs on the RHS.
                        right.meta.disabled = false;
                    }
                }
            },
            onObject: onObjectOrArray,
            onArray: onObjectOrArray
        });
    }

    return {
        processComparisonObjects: walk
    }
});