cmsApp.factory("schemaWalker", function () {

    function delegate(member, metaLeft, metaRight, callbacks){

        switch(member.type){
            case "object":
                if ((typeof metaLeft != "object" || metaLeft instanceof Array) && typeof metaLeft != "undefined"){
                    throw new Error("json value is not of type 'object'");
                }
                if ((typeof metaRight != "object" || metaRight instanceof Array) && typeof metaRight != "undefined"){
                    throw new Error("json value is not of type 'object'");
                }
                callbacks.onAll(member, metaLeft, metaRight);
                handleObject(member, metaLeft, metaRight, callbacks);
                break;
            case "array":
                if (typeof metaLeft != "object" && typeof metaLeft != "undefined"){
                    throw new Error("json value is not of type 'array'");
                }
                if (typeof metaLeft == "object" && !(metaLeft instanceof Array) ){
                    throw new Error("json value is not of type 'array'");
                }

                if (typeof metaRight != "object" && typeof metaRight != "undefined" && !(metaRight instanceof Array)){
                    throw new Error("json value is not of type 'array'");
                }
                if (typeof metaRight == "object" && !(metaRight instanceof Array) ){
                    throw new Error("json value is not of type 'array'");
                }
                callbacks.onAll(member, metaLeft, metaRight);
                handleArray(member, metaLeft, metaRight, callbacks);
                break;
            case "string":
                 if (metaLeft != undefined && metaLeft.value != undefined && typeof metaLeft.value != "string"){
                    throw new Error("left json value '" + metaLeft.value +  "' is not of type 'string'");
                }

                if (metaRight != undefined && metaRight.value != undefined && typeof metaRight.value != "string"){
                    throw new Error("right json value '" + metaRight.value +  "'  is not of type 'string'");
                }
                callbacks.onAll(member, metaLeft, metaRight);
                handlePrimitive(member, metaLeft, metaRight, callbacks);
                break;
            default:
                throw new Error("element type in schema is '" + member.type + "'")
        }
    }

    function walkObject(schema, jsonLeft, jsonRight, callbacks){
        for (var property in schema.properties) {
            var member = schema.properties[property];
            var leftValue = ndef(jsonLeft, property);
            var rightValue = ndef(jsonRight, property);

            delegate(member, leftValue, rightValue, callbacks);
        }
    }

    function handleObject(schema, objectLeft, objectRight, callbacks) {
        if (schema.properties == undefined){
            throw new Error("element 'properties' of object is not defined");
        }

        callbacks.onObject(schema, objectLeft, objectRight);

        walkObject(schema, objectLeft, objectRight, callbacks);
    }

    function handleArray(schema, arrayLeft, arrayRight, callbacks){
        if (schema.items == undefined){
            throw new Error("element 'items' of object is not defined");
        }

        callbacks.onArray(schema, arrayLeft, arrayRight);

        var length = Math.max( arrayLeft == undefined ? 0 : arrayLeft.length, arrayRight == undefined ? 0 : arrayRight.length);

        for (var i = 0; i < length; i++ ) {
            delegate(schema.items, ndef(arrayLeft, i), ndef(arrayRight, i), callbacks);
        }
    }

    function handlePrimitive(schema, left, right, callbacks) {
        callbacks.onPrimitive(schema, left, right);
    }

    function ndef(collection, key){
        if (collection == undefined){
            return undefined;
        } else {
            return collection[key];
        }
    }

    return {
        walk: delegate
    }
});